// KittyCat! Comm 1.1 for Windows 3.1
// by Steven Lawrance
// ---KittyDDE---

#include <windows.h>
#include <string.h>
#include <ctl3d.h>
#include <stdlib.h>
#include "ktydders.h"
#include "Structs.h"

#define MAXDOOR 30
#define MAXMODEM 8
#define NAMELEN 10
#define MAXCOMPUTER 8

// Global Variables
typedef struct tagDoorDataStruct
{
  HDOOR 			hRemoteDoor[MAXCOMPUTER];
  BOOL         bTokenStatus[MAXMODEM];
  BOOL         bComputerToken[MAXCOMPUTER];
  char  			szProgramName[NAMELEN];
  KITTYDDEPROC fnCallBack;
  UINT			Flags;
  BOOL         bTokenRequest;
  BOOL		   bInUse;
  BOOL		   bHalt;
} DoorDataStruct;

typedef struct tagComputerDataStruct
{
  HCOMPUTER    hGlobalComputer;
  HMODEM		   hModem;
  char  			szComputerName[NAMELEN];
} ComputerDataStruct;

typedef struct tagModemDataStruct
{
  // General
  HWND			hDlg;
  BOOL         bModemEscape;
  HDOOR        hToken;
  unsigned int iQueueSize;
  int				hComm;

  // Receive buffer
  HGLOBAL	    lpData;
  HDOOR		    hDoor;
  unsigned long Crc;
  UINT 			 nBytes;
  char          fType;

  // Last transfer information
  BOOL			bLastTransfer;
  BOOL         bLastTransferAltDone;
  HGLOBAL      lpLastTransfer;
  HDOOR			hLastTransferDoor;
  char  			fLastTransferType;
} ModemDataStruct;

// Lists
DoorDataStruct 	  Door[MAXDOOR];
ComputerDataStruct  Computer[MAXCOMPUTER];
ModemDataStruct     Modem[MAXMODEM];

// Misc variables
HINSTANCE      	  ghInstance;
DLGPROC				  hKittyDdeDlgProc;
BOOL		           bSoundFlag;
BOOL   				  bRenameComputer;
HDOOR					  hNewDoor;
HMODEM				  hNewModem;


// KittyDde Functions
HDOOR FAR PASCAL _export KittyDdeCreateDoor(LPCSTR lpszProgramName, UINT Flags, KITTYDDEPROC fnCallBack);
BOOL FAR PASCAL _export KittyDdeDeleteDoor(HDOOR hDoor);
BOOL FAR PASCAL _export KittyDdeIsTokenFree(HCOMPUTER hComputer);
BOOL FAR PASCAL _export KittyDdeGetAllTokens(HDOOR hDoor);
BOOL FAR PASCAL _export KittyDdeGetOneToken(HDOOR hDoor, HCOMPUTER hComputer);
BOOL FAR PASCAL _export KittyDdeGetTokenList(HDOOR hDoor, int nItems, HCOMPUTER FAR * hComputer);
void FAR PASCAL _export KittyDdeFreeTokens(HDOOR hDoor);
HCOMPUTER FAR PASCAL _export KittyDdeGetHComputer(LPCSTR lpszComputerName);
void FAR PASCAL _export KittyDdeGetComputerList(LPSTR FAR * lpszComputerName, UINT iMax);
UINT FAR PASCAL _export KittyDdeGetNumComputers(void);
BOOL FAR PASCAL _export KittyDdeGetComputerName(HCOMPUTER hComputer, LPSTR lpszComputerName);
BOOL FAR PASCAL _export KittyDdeHaltIncomingData(HDOOR hDoor, UINT fHaltFlag);
HINSTANCE FAR PASCAL _export KittyDdeGetInstance(void);
BOOL FAR PASCAL _export KittyDdeGetSoundFlag(void);
void FAR PASCAL _export KittyDdeFreeTokenBuffer(HDOOR hDoor, UINT nBytes);
void FAR PASCAL _export KittyDdeFreeResend(HDOOR hDoor);
BOOL FAR PASCAL _export KittyDdeTerminateRemoteDoor(HDOOR hDoor, HCOMPUTER hComputer);
UINT FAR PASCAL _export KittyDdeGetFlags(HDOOR hDoor);
BOOL FAR PASCAL _export KittyDdeSetFlags(HDOOR hDoor, UINT Flags);
unsigned long FAR PASCAL _export KittyDdeCalculateKittyCrc32(LPBYTE lpData, UINT nBytes);
BOOL FAR PASCAL _export KittyDdeSendData(HDOOR hDoor, LPBYTE lpData, UINT nBytes);
BOOL FAR PASCAL _export KittyDdeSendText(HDOOR hDoor, LPCSTR lpszData);
BOOL FAR PASCAL _export KittyDdeSendValue(HDOOR hDoor, LPBYTE lpData, char message);
BOOL FAR PASCAL _export KittyDdeSendChar(HDOOR hDoor, char cData);
BOOL FAR PASCAL _export KittyDdeSendUnsignedChar(HDOOR hDoor, unsigned char cData);
BOOL FAR PASCAL _export KittyDdeSendInteger(HDOOR hDoor, int iData);
BOOL FAR PASCAL _export KittyDdeSendUnsignedInteger(HDOOR hDoor, UINT iData);
BOOL FAR PASCAL _export KittyDdeSendLong(HDOOR hDoor, long lData);
BOOL FAR PASCAL _export KittyDdeSendUnsignedLong(HDOOR hDoor, unsigned long lData);
BOOL FAR PASCAL _export KittyDdeSendFloat(HDOOR hDoor, float fData);
BOOL FAR PASCAL _export KittyDdeSendDouble(HDOOR hDoor, double dData);
BOOL FAR PASCAL _export KittyDdeSendLongDouble(HDOOR hDoor, long double ldData);
HCOMPUTER FAR PASCAL _export KittyDdeGetNextHComputer(HCOMPUTER hComputer);

// Internal Functions
BOOL FAR PASCAL _export InternalGetToken(HMODEM hModem);
void FAR PASCAL _export InternalFreeToken(HMODEM hModem);
HMODEM FAR PASCAL _export InternalNewModem(BBS bbs, UINT iQueueSize);
void FAR PASCAL _export InternalSetComputer(LPCSTR lpszName);
void FAR PASCAL _export InternalSetSoundFlag(BOOL bNewSoundFlag);
BOOL FAR PASCAL _export InternalSendModemEscape(HMODEM hModem, LPCSTR lpszString, BOOL bReturnToken);
HMODEM FAR PASCAL _export InternalGetHModem(HWND hDlg);
void FAR PASCAL _export InternalDeleteModem(HMODEM hModem);
UINT FAR PASCAL _export InternalGetLength(char cType);
UINT FAR PASCAL _export InternalFreeTokenBuffer(HMODEM hModem, UINT nBytes);
void FAR PASCAL _export InternalFreeResend(HMODEM hModem);
void FAR PASCAL _export InternalSendBroadcastMessage(char message, HDOOR hDoor, LPBYTE lpData, UINT nBytes);
int FAR PASCAL _export InternalOpenPort(BBS bbsx, UINT iQueueSize);

// Global Functions
BOOL CALLBACK _export KittyDdeDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
void FAR PASCAL _export Initialize(BOOL bStart);
BOOL SendDoorMessage(HDOOR hDoor, char message, HCOMPUTER hComputer, LPBYTE lpData, WPARAM wParam, LPARAM lParam);

#pragma argsused
int CALLBACK LibMain (HINSTANCE hInstance, WORD wDataSeg, WORD wHeapSize, LPSTR lpszCmdLine)
{
  int i, t;
  if (wHeapSize > 0)
	 UnlockData(0);

  ghInstance = hInstance;
  hKittyDdeDlgProc = NULL;
  bSoundFlag = FALSE;
  bRenameComputer = FALSE;
  hNewDoor = HDOOR_NO_REMOTE_DOOR;
  hNewModem = -1;
  for (i = 0; i <= MAXMODEM - 1; i++)
  {
	 Modem[i].hDlg = NULL;
	 Modem[i].bModemEscape = FALSE;
	 Modem[i].hToken = HTOKEN_NA;
	 Modem[i].hComm = -1;
	 Modem[i].iQueueSize = 0;

	 Modem[i].lpData = NULL;
	 Modem[i].hDoor = HDOOR_NO_REMOTE_DOOR;
	 Modem[i].Crc = 0;
	 Modem[i].nBytes = 0;
	 Modem[i].fType = KITTYDDE_OK;

	 Modem[i].bLastTransfer = FALSE;
	 Modem[i].bLastTransferAltDone = FALSE;
	 Modem[i].lpLastTransfer = NULL;
	 Modem[i].hLastTransferDoor = HDOOR_NO_REMOTE_DOOR;
	 Modem[i].fLastTransferType = KITTYDDE_OK;
  }
  for (i = 0; i <= MAXCOMPUTER - 1; i++)
  {
	 Computer[i].hGlobalComputer = 0;
	 Computer[i].hModem = -1;
	 memset(Computer[i].szComputerName, NULL, NAMELEN);
  }
  for (i = 0; i <= MAXDOOR - 1; i++)
  {
	 for (t = 0; t <= MAXCOMPUTER - 1; t++)
		Door[i].hRemoteDoor[t] = HDOOR_NO_REMOTE_DOOR;
	 for (t = 0; t <= MAXMODEM - 1; t++)
		Door[i].bTokenStatus[t] = HTOKEN_NA;
	 for (t = 0; t <= MAXCOMPUTER - 1; t++)
		Door[i].bComputerToken[t] = HTOKEN_NA;
	 memset(Door[i].szProgramName, NULL, NAMELEN);
	 Door[i].fnCallBack = NULL;
	 Door[i].Flags = 0;
	 Door[i].bTokenRequest = FALSE;
	 Door[i].bInUse = FALSE;
	 Door[i].bHalt = FALSE;
  }

//  Ctl3dRegister(ghInstance);
//  Ctl3dAutoSubclass(ghInstance);
  hKittyDdeDlgProc = (DLGPROC) MakeProcInstance((FARPROC) KittyDdeDlgProc, ghInstance);
  return 1;
}

#pragma argsused
int CALLBACK WEP(int nExitType)
{
  FreeProcInstance((FARPROC) hKittyDdeDlgProc);
//  Ctl3dUnregister(ghInstance);
  return 1;
}

HDOOR FAR PASCAL _export KittyDdeCreateDoor(LPCSTR lpszProgramName, UINT Flags, KITTYDDEPROC fnCallBack)
{
  HDOOR hDoor;
  BOOL bSuccess = FALSE;
  int i;

  // If another new door is being registered, return HDOOR_BUSY_REGISTERING
  if (hNewDoor >= 0)
	 return HDOOR_BUSY_REGISTERING;

  // Look for Empty Address
  for (i = 0; (i <= MAXDOOR - 1) && (!bSuccess); i++)
  {
	 if (Door[i].bInUse == FALSE)
	 {
		hDoor = i;
		bSuccess = TRUE;
	 }
  }

  // If no success, assign appropriate error message to hDoor
  if (!bSuccess)
	 bSuccess = HDOOR_NO_DOORS_LEFT;

  // If success, then assign variables to passed values
  else
  {
	 // Reset door
	 for (i = 0; i <= MAXCOMPUTER - 1; i++)
	 {
		Door[hDoor].hRemoteDoor[i] = HDOOR_NO_REMOTE_DOOR;
		if (Computer[i].hModem >= 0)
		  Door[hDoor].bComputerToken[i] = HTOKEN_FREE;
		else
		  Door[hDoor].bComputerToken[i] = HTOKEN_NA;
	 }
	 for (i = 0; i <= MAXMODEM - 1; i++)
	 {
		if (Modem[i].hDlg != NULL)
		  Door[hDoor].bTokenStatus[i] = HTOKEN_FREE;
		else
		  Door[hDoor].bTokenStatus[i] = HTOKEN_NA;
	 }
	 memset(Door[hDoor].szProgramName, NULL, NAMELEN);
	 Door[hDoor].fnCallBack = NULL;
	 Door[hDoor].Flags = 0;
	 Door[hDoor].bTokenRequest = FALSE;
	 Door[hDoor].bInUse = FALSE;
	 Door[hDoor].bHalt = FALSE;

	 // Assign values
	 strncpy(Door[hDoor].szProgramName, lpszProgramName, NAMELEN);
	 Door[hDoor].fnCallBack = fnCallBack;
	 Door[hDoor].Flags = Flags;
	 Door[hDoor].bInUse = TRUE;
	 Door[hDoor].bHalt = FALSE;
	 hNewDoor = hDoor;
  }
  return hDoor;
}

BOOL FAR PASCAL _export KittyDdeDeleteDoor(HDOOR hDoor)
{
  // Invalidate door handle
  if (Door[hDoor].bInUse == TRUE)
  {
	 Door[hDoor].bInUse = FALSE;
	 return TRUE;
  }
  return HDOOR_BADHANDLE;
}

BOOL FAR PASCAL _export KittyDdeIsTokenFree(HCOMPUTER hComputer)
{
  if (Modem[Computer[hComputer].hModem].hToken == HTOKEN_FREE)
	 return TRUE;
  return FALSE;
}

BOOL FAR PASCAL _export KittyDdeGetAllTokens(HDOOR hDoor)
{
  // Request all tokens for given door
  HMODEM hModem;
  BOOL bSuccess;
  int i;
  for (i = 0; i <= MAXCOMPUTER - 1; i++)
  {
	 // Assign token to computers that the same door is running on
	 hModem = Computer[i].hModem;
	 bSuccess = FALSE;
	 if ((Modem[hModem].hToken == HTOKEN_FREE) && (Door[hDoor].hRemoteDoor[i] != HDOOR_NO_REMOTE_DOOR) && (!bSuccess))
	 {
		Door[hDoor].bTokenStatus[hModem] = HTOKEN_LOCKED;
		Modem[hModem].hToken = hDoor;
		Door[hDoor].bTokenRequest = TRUE;
		bSuccess = TRUE;
		Door[hDoor].bComputerToken[i] = HTOKEN_LOCKED;
	 }
	 if ((Modem[hModem].hToken != HTOKEN_FREE) && (Door[hDoor].hRemoteDoor[i] != HDOOR_NO_REMOTE_DOOR) && (!bSuccess) && (Modem[hModem].hToken != hDoor))
	 {
		Door[hDoor].bTokenStatus[hModem] = HTOKEN_WAITING;
		Door[hDoor].bTokenRequest = TRUE;
		do {
		  WaitMessage();
		}	while ((Door[hDoor].bTokenRequest == TRUE) && (Modem[hModem].hToken != HTOKEN_FREE));
		if (Modem[hModem].hToken != HTOKEN_FREE)
		  return HTOKEN_ERROR;
		else
		{
		  Door[hDoor].bTokenStatus[hModem] = HTOKEN_LOCKED;
		  Modem[hModem].hToken = hDoor;
		  Door[hDoor].bComputerToken[i] = HTOKEN_LOCKED;
		}
		bSuccess = TRUE;
	 }
  }
  return TRUE;
}

BOOL FAR PASCAL _export KittyDdeGetOneToken(HDOOR hDoor, HCOMPUTER hComputer)
{
  // Request one token for given door
  HMODEM hModem = Computer[hComputer].hModem;
  BOOL bSuccess = FALSE;

  if ((Modem[hModem].hToken == HTOKEN_FREE) && (Door[hDoor].hRemoteDoor[hComputer] != HDOOR_NO_REMOTE_DOOR) && (!bSuccess))
  {
	 Door[hDoor].bTokenStatus[hModem] = HTOKEN_LOCKED;
	 Modem[hModem].hToken = hDoor;
	 Door[hDoor].bTokenRequest = TRUE;
	 Door[hDoor].bComputerToken[hComputer] = HTOKEN_LOCKED;
	 bSuccess = TRUE;
  }
  if ((Modem[hModem].hToken != HTOKEN_FREE) && (Door[hDoor].hRemoteDoor[hComputer] != HDOOR_NO_REMOTE_DOOR) && (!bSuccess) && (Modem[hModem].hToken != hDoor))
  {
	 Door[hDoor].bTokenStatus[hModem] = HTOKEN_WAITING;
	 Door[hDoor].bTokenRequest = TRUE;
	 do {
		WaitMessage();
	 }	while ((Door[hDoor].bTokenRequest == TRUE) && (Modem[hModem].hToken != HTOKEN_FREE));
	 if (Modem[hModem].hToken != HTOKEN_FREE)
		return HTOKEN_ERROR;
	 else
	 {
		Door[hDoor].bTokenStatus[hModem] = HTOKEN_LOCKED;
		Modem[hModem].hToken = hDoor;
		Door[hDoor].bComputerToken[hComputer] = HTOKEN_LOCKED;
	 }
	 bSuccess = TRUE;
  }
  if ((Door[hDoor].hRemoteDoor[hComputer] == HDOOR_NO_REMOTE_DOOR) && (!bSuccess))
	 return HTOKEN_BADHCOMPUTER;
  return TRUE;
}

BOOL FAR PASCAL _export KittyDdeGetTokenList(HDOOR hDoor, int nItems, HCOMPUTER FAR * hComputer)
{
  // Request tokens from a list for given door
  HMODEM hModem;
  BOOL bSuccess;
  int i;

  for (i = 0; i <= nItems - 1; i++)
  {
	 bSuccess = FALSE;
	 hModem = Computer[hComputer[i]].hModem;
	 if ((Modem[hModem].hToken == HTOKEN_FREE) && (Door[hDoor].hRemoteDoor[hComputer[i]] != HDOOR_NO_REMOTE_DOOR) && (!bSuccess))
	 {
		Door[hDoor].bTokenStatus[hModem] = HTOKEN_LOCKED;
		Modem[hModem].hToken = hDoor;
		Door[hDoor].bTokenRequest = TRUE;
		Door[hDoor].bComputerToken[hComputer[i]] = HTOKEN_LOCKED;
		bSuccess = TRUE;
	 }
	 if ((Modem[hModem].hToken != HTOKEN_FREE) && (Door[hDoor].hRemoteDoor[hComputer[i]] != HDOOR_NO_REMOTE_DOOR) && (!bSuccess) && (Modem[hModem].hToken != hDoor))
	 {
		Door[hDoor].bTokenStatus[hModem] = HTOKEN_WAITING;
		Door[hDoor].bTokenRequest = TRUE;
		do {
		  WaitMessage();
		}	while ((Door[hDoor].bTokenRequest == TRUE) && (Modem[hModem].hToken != HTOKEN_FREE));
		if (Modem[hModem].hToken != HTOKEN_FREE)
		  return HTOKEN_ERROR;
		else
		{
		  Door[hDoor].bTokenStatus[hModem] = HTOKEN_LOCKED;
		  Modem[hModem].hToken = hDoor;
		  Door[hDoor].bComputerToken[hComputer[i]] = HTOKEN_LOCKED;
		}
		bSuccess = TRUE;
	 }
	 if ((Door[hDoor].hRemoteDoor[hComputer[i]] == HDOOR_NO_REMOTE_DOOR) && (!bSuccess))
	 {
		KittyDdeFreeTokens(hDoor);
		return HMODEM_BADHANDLE;
	 }
  }
  return TRUE;
}

void FAR PASCAL _export KittyDdeFreeTokens(HDOOR hDoor)
{
  // Frees all the tokens that the given door has in custody
  int i;
  Door[hDoor].bTokenRequest = FALSE;
  for (i = 0; i <= MAXMODEM - 1; i++)
  {
	 if (Door[hDoor].bTokenStatus[i] == HTOKEN_LOCKED)
	 {
		// Unlock global token and free local door token
		if (Modem[i].hToken != hDoor)
		{
		  MessageBeep(MB_ICONSTOP);
		  MessageBox(Modem[i].hDlg, "Token inconsistancy error!  I will free the token that attempted to be freed.  Please exit your KittyDde programs as soon as possible!", "Free Token", MB_OK | MB_ICONSTOP);
		}
		Modem[i].hToken = HTOKEN_FREE;
		Door[hDoor].bTokenStatus[i] = HTOKEN_FREE;
	 }
	 if (Door[hDoor].bTokenStatus[i] == HTOKEN_WAITING)
	 {
		// Free the local door token
		Door[hDoor].bTokenStatus[i] = HTOKEN_FREE;
	 }
  }
  for (i = 0; i <= MAXCOMPUTER - 1; i++)
	 Door[hDoor].bComputerToken[i] = HTOKEN_FREE;
}

HCOMPUTER FAR PASCAL _export KittyDdeGetHComputer(LPCSTR lpszComputerName)
{
  // Search for computer name and return handle, if any
  HCOMPUTER hComputer = -1;
  int i;
  for (i = 0; (i <= MAXCOMPUTER - 1) && (hComputer == -1); i++)
	 if (strcmp(Computer[i].szComputerName, (char *) lpszComputerName))
		hComputer = i;
  return hComputer;
}

void FAR PASCAL _export KittyDdeGetComputerList(LPSTR FAR * lpszComputerName, UINT iMax)
{
  // Get computer list
  UINT i = 0, t = 0;
  while ((i <= iMax - 1) && (t <= MAXCOMPUTER - 1))
  {
	 while ((t <= MAXCOMPUTER - 1) && (Computer[t].hModem < 0))
		t++;
	 if (t <= MAXCOMPUTER - 1)
		lstrcpy(lpszComputerName[i], Computer[t].szComputerName);
	 i++;
	 t++;
  }
}

UINT FAR PASCAL _export KittyDdeGetNumComputers(void)
{
  // Get the number of computers
  UINT i = 0, NumComputers = 0;
  for (i = 0; i <= MAXCOMPUTER - 1; i++)
	 if (Computer[i].hModem >= 0)
		NumComputers++;
  return NumComputers;
}

BOOL FAR PASCAL _export KittyDdeGetComputerName(HCOMPUTER hComputer, LPSTR lpszComputerName)
{
  // Get the name of the computer with the given handle
  if (hComputer >= MAXCOMPUTER)
	 return FALSE;
  lstrcpy(lpszComputerName, Computer[hComputer].szComputerName);
  return TRUE;
}

BOOL FAR PASCAL _export KittyDdeHaltIncomingData(HDOOR hDoor, UINT fHaltFlag)
{
  // Halt or resume incoming data for that door
  if (fHaltFlag == KITTYDDE_WAIT)
	 Door[hDoor].bHalt = TRUE;
  if (fHaltFlag == KITTYDDE_RESUME)
	 Door[hDoor].bHalt = FALSE;
  return Door[hDoor].bHalt;
}

HINSTANCE FAR PASCAL _export KittyDdeGetInstance(void)
{
  // Return instance handle
  return ghInstance;
}

BOOL FAR PASCAL _export KittyDdeGetSoundFlag(void)
{
  // Return the sound flag
  return bSoundFlag;
}

BOOL FAR PASCAL _export InternalGetToken(HMODEM hModem)
{
  // Get token for internal usage
  if (Modem[hModem].hToken == HTOKEN_FREE)
  {
	 // Horray!  Token was free in the first place!
	 Modem[hModem].hToken = HTOKEN_INTERNAL;
  }
  if ((Modem[hModem].hToken != HTOKEN_FREE) && (Modem[hModem].hToken != HTOKEN_INTERNAL))
  {
	 // Oh dear!  We'll need to wait until the token is free.
	 do {
		WaitMessage();
	 } while ((Modem[hModem].hToken != HTOKEN_FREE) && (Modem[hModem].hToken != HTOKEN_ERROR));
	 if (Modem[hModem].hToken != HTOKEN_FREE)
		return HTOKEN_ERROR;
	 else
		Modem[hModem].hToken = HTOKEN_INTERNAL;
  }
  return TRUE;
}

void FAR PASCAL _export InternalFreeToken(HMODEM hModem)
{
  // Free the token that is being used internally, if any
  if (Modem[hModem].hToken == HTOKEN_INTERNAL)
	 Modem[hModem].hToken = HTOKEN_FREE;
}

void FAR PASCAL _export KittyDdeFreeTokenBuffer(HDOOR hDoor, UINT nBytes)
{
  // Free up buffer storage for all locked tokens of given door
  COMSTAT cs;
  int i;
  for (i = 0; i <= MAXMODEM - 1; i++)
  {
	 // If the token for this modem is locked, wait until the space is free
	 if ((Door[hDoor].bTokenStatus[i] == HTOKEN_LOCKED) && (Modem[i].hToken == hDoor))
	 {
		GetCommError(Modem[i].hComm, &cs);
		while (Modem[i].iQueueSize - cs.cbOutQue < nBytes)
		{
		  WaitMessage();
		  GetCommError(Modem[i].hComm, &cs);
		}
	 }
  }
}

void FAR PASCAL _export KittyDdeFreeResend(HDOOR hDoor)
{
  // Wait until the resend buffer is free
  int i;
  for (i = 0; i <= MAXMODEM - 1; i++)
  {
	 // If the token for this modem is locked, wait until the resend is free
	 if ((Door[hDoor].bTokenStatus[i] == HTOKEN_LOCKED) && (Modem[i].hToken == hDoor))
		while (Modem[i].bLastTransfer == TRUE)
		  WaitMessage();
  }
}

BOOL FAR PASCAL _export KittyDdeTerminateRemoteDoor(HDOOR hDoor, HCOMPUTER hComputer)
{
  // Terminate remote door on hComputer
  int hComm = Modem[Computer[hComputer].hModem].hComm;

  // Handle exception
  if (Door[hDoor].hRemoteDoor[hComputer] == HDOOR_NO_REMOTE_DOOR)
	 return HDOOR_NO_REMOTE_DOOR;

  // Get token and free up space
  InternalGetToken(Computer[hComputer].hModem);
  KittyDdeFreeTokenBuffer(hComputer, 6);
  KittyDdeFreeResend(hComputer);

  // Transmit information
  TransmitCommChar(hComm, (char) KITTYDDE_TERMINATEDOOR);
  WriteComm(hComm, (LPBYTE) Computer[hComputer].hGlobalComputer, 2);
  WriteComm(hComm, (LPBYTE) Door[hDoor].hRemoteDoor[hComputer], 2);
  TransmitCommChar(hComm, (char) KITTYDDE_DONE);

  // Free internal token
  InternalFreeToken(Computer[hComputer].hModem);

  // Set last transfer information
  Modem[Computer[hComputer].hModem].bLastTransfer = TRUE;
  Modem[Computer[hComputer].hModem].hLastTransferDoor = hDoor;
  Modem[Computer[hComputer].hModem].fLastTransferType = KITTYDDE_TERMINATEDOOR;
  return TRUE;
}

UINT FAR PASCAL _export KittyDdeGetFlags(HDOOR hDoor)
{
  // Return the door flags even if handle is invalid
  return Door[hDoor].Flags;
}

BOOL FAR PASCAL _export KittyDdeSetFlags(HDOOR hDoor, UINT Flags)
{
  // Set door flags if the door is valid
  if (Door[hDoor].bInUse == TRUE)
  {
	 Door[hDoor].Flags = Flags;
	 return TRUE;
  }
  return HDOOR_BADHANDLE;
}

BOOL SendDoorMessage(HDOOR hDoor, char message, HCOMPUTER hComputer, LPBYTE lpData, WPARAM wParam, LPARAM lParam)
{
  // Sends a message to a door
  if (Door[hDoor].bInUse)
  {
	 // Check flags regarding message
	 if ( ((Door[hDoor].Flags & DOOR_NO_SOUND_NOTIFY) && (message == KITTYDDE_NEWSOUNDFLAG)) ||
			((Door[hDoor].Flags & DOOR_NO_RESUME_NOTIFY) && (message == KITTYDDE_RESUME)) ||
			((Door[hDoor].Flags & DOOR_NO_BUSY_NOTIFY) && (message == KITTYDDE_WAIT)) )
		return HDOOR_FLAGDISABLE;

	 // Send message to door's callback procedure
	 if (Door[hDoor].fnCallBack != NULL)
		return (BOOL) ((*Door[hDoor].fnCallBack) (hDoor, message, hComputer, lpData, wParam, lParam));
  }
  return HDOOR_BADHANDLE;
}

HMODEM FAR PASCAL _export InternalNewModem(BBS bbs, UINT iQueueSize)
{
  // Initialize a new modem
  BOOL bSuccess = FALSE;
  HMODEM hModem;
  int i, hComm;

  // If it is currently registering another modem, return a wait
  if (hNewModem >= 0)
	 return HMODEM_COMMBUSY;

  // Open communications port
  hComm = InternalOpenPort(bbs, iQueueSize);
  if (hComm < 0)
	 return HMODEM_BADHANDLE;

  // Search for an available one and, if no available port is found, return an error
  for (i = 0; (i <= MAXMODEM - 1) && (!bSuccess); i++)
  {
	 if (Modem[i].hComm < 0)
	 {
		Modem[i].hDlg = NULL;
		Modem[i].hToken = HTOKEN_FREE;
		Modem[i].iQueueSize = iQueueSize;
		Modem[i].hComm = hComm;

		Modem[i].lpData = NULL;
		Modem[i].hDoor = HDOOR_NO_REMOTE_DOOR;
		Modem[i].Crc = 0;
		Modem[i].nBytes = 0;
		Modem[i].fType = KITTYDDE_OK;

		Modem[i].bLastTransfer = FALSE;
		Modem[i].bLastTransferAltDone = FALSE;
		Modem[i].lpLastTransfer = NULL;
		Modem[i].hLastTransferDoor = HDOOR_NO_REMOTE_DOOR;
		Modem[i].fLastTransferType = KITTYDDE_OK;
		bSuccess = TRUE;
		hModem = i;
	 }
  }
  if (!bSuccess)
	 return HMODEM_NO_MODEMS_LEFT;
  hNewModem = hModem;
  Modem[hModem].hDlg = CreateDialog(ghInstance, "DDE", HWND_DESKTOP, hKittyDdeDlgProc);
  return hModem;
}

void FAR PASCAL _export InternalSetComputer(LPCSTR lpszName)
{
  // Name computer
  memset(Computer[0].szComputerName, NULL, NAMELEN);
  strcpy(Computer[0].szComputerName, (char *) lpszName);
  bRenameComputer = TRUE;
}

unsigned long FAR PASCAL _export KittyDdeCalculateKittyCrc32(LPBYTE lpData, UINT nBytes)
{
  // Calculate KittyCrc32
  unsigned long Crc = 1;
  unsigned int i;
  for (i = 0; i <= nBytes - 1; i++)
	 Crc = Crc + ((i + 1) * lpData[i]);
  return Crc;
}

void FAR PASCAL _export InternalSetSoundFlag(BOOL bNewSoundFlag)
{
  // Set sound flag and notify doors about change if changed
  if (bSoundFlag != bNewSoundFlag)
  {
	 int i;
	 bSoundFlag = bNewSoundFlag;
	 for (i = 0; i <= MAXDOOR - 1; i++)
	 {
		if (Door[i].bInUse)
		  SendDoorMessage(i, KITTYDDE_NEWSOUNDFLAG, -1, NULL, NULL, (LPARAM) bSoundFlag);
	 }
  }
}

BOOL FAR PASCAL _export KittyDdeSendData(HDOOR hDoor, LPBYTE lpData, UINT nBytes)
{
  // Send data (byte form) to current token(s)
  LPBYTE *lpResendBuffer;
  BOOL bSuccess = KITTYDDE_NO_TOKEN;
  int i, t;

  // Send a packet for each modem that the computer token is locked to
  for (i = 0; i <= MAXMODEM - 1; i++)
  {
	 HCOMPUTER hComputer[MAXCOMPUTER];
	 HDOOR hRemoteDoor[MAXCOMPUTER];
	 int iNextComputer = 0;
	 hComputer[0] = -1;
	 hRemoteDoor[0] = -1;

	 // Determine which computer tokens are locked to current modem to prevent
	 // redundant transmittion to same modem for same packet  :-)
	 for (t = 0; t <= MAXCOMPUTER - 1; t++)
	 {
		if ((Door[hDoor].bComputerToken[t] == HTOKEN_LOCKED) && (Door[hDoor].bTokenStatus[i] == HTOKEN_LOCKED) && (Computer[t].hModem == i) && (Door[hDoor].hRemoteDoor[t] != HDOOR_NO_REMOTE_DOOR))
		{
		  Door[hDoor].bComputerToken[t] = HTOKEN_UNLOCKED;
		  hRemoteDoor[iNextComputer] = Door[hDoor].hRemoteDoor[t];
		  hComputer[iNextComputer] = Computer[t].hGlobalComputer;
		  if (iNextComputer < MAXCOMPUTER - 1)
			 iNextComputer++;
		}
	 }

	 // If there are computers that use this modem, then send one packet for
	 // all the computers connected to current modem
	 if ((hComputer[0] != -1) && (!Modem[i].bLastTransfer))
	 {
		// Message type
		bSuccess = KITTYDDE_OK;
		Modem[i].lpLastTransfer = GlobalAlloc(GMEM_MOVEABLE | GMEM_ZEROINIT, nBytes);
		TransmitCommChar(Modem[i].hComm, (char) KITTYDDE_DATA);

		// Send hComputer and hDoor information
		if (iNextComputer - 1 > 0)
		  TransmitCommChar(Modem[i].hComm, (char) KITTYDDE_HCOMPUTERSTRING);
		for (t = 0; t <= iNextComputer - 1; t++)
		{
		  WriteComm(Modem[i].hComm, (LPBYTE) hComputer[t], 2);
		  WriteComm(Modem[i].hComm, (LPBYTE) hRemoteDoor[t], 2);
		}
		if (iNextComputer - 1 > 0)
		  TransmitCommChar(Modem[i].hComm, (char) KITTYDDE_HCOMPUTERDONE);

		// Send KittyCrc32 and data fork length
		WriteComm(Modem[i].hComm, (LPBYTE) KittyDdeCalculateKittyCrc32(lpData, nBytes), 4);
		WriteComm(Modem[i].hComm, (LPBYTE) nBytes, 2);

		// Send data fork and store it in the resend buffer
		*lpResendBuffer = GlobalLock(Modem[i].lpLastTransfer);
		WriteComm(Modem[i].hComm, lpData, nBytes);
		*lpResendBuffer = lpData;
		GlobalUnlock(Modem[i].lpLastTransfer);

		// Send termination byte
		TransmitCommChar(Modem[i].hComm, (char) KITTYDDE_DONE);

		// Set transmittion variables
		Modem[i].bLastTransfer = TRUE;
		Modem[i].hLastTransferDoor = hDoor;
		Modem[i].fLastTransferType = KITTYDDE_DATA;
	 }
	 if (Modem[i].bLastTransfer)
		bSuccess = KITTYDDE_BUSY;
  }
  return bSuccess;
}

BOOL FAR PASCAL _export KittyDdeSendText(HDOOR hDoor, LPCSTR lpszData)
{
  // Send data (byte form) to current token(s)
  LPSTR lpszResendBuffer;
  BOOL bSuccess = KITTYDDE_NO_TOKEN;
  int i, t;

  // Send a packet for each modem that the computer token is locked to
  for (i = 0; i <= MAXMODEM - 1; i++)
  {
	 HCOMPUTER hComputer[MAXCOMPUTER];
	 HDOOR hRemoteDoor[MAXCOMPUTER];
	 int iNextComputer = 0;
	 hComputer[0] = -1;
	 hRemoteDoor[0] = -1;

	 // Determine which computer tokens are locked to current modem to prevent
	 // redundant transmittion to same modem for same packet  :-)
	 for (t = 0; t <= MAXCOMPUTER - 1; t++)
	 {
		if ((Door[hDoor].bComputerToken[t] == HTOKEN_LOCKED) && (Door[hDoor].bTokenStatus[i] == HTOKEN_LOCKED) && (Computer[t].hModem == i) && (Door[hDoor].hRemoteDoor[t] != HDOOR_NO_REMOTE_DOOR))
		{
		  Door[hDoor].bComputerToken[t] = HTOKEN_UNLOCKED;
		  hRemoteDoor[iNextComputer] = Door[hDoor].hRemoteDoor[t];
		  hComputer[iNextComputer] = Computer[t].hGlobalComputer;
		  if (iNextComputer < MAXCOMPUTER - 1)
			 iNextComputer++;
		}
	 }

	 // If there are computers that use this modem, then send one packet for
	 // all the computers connected to current modem
	 if ((hComputer[0] != -1) && (!Modem[i].bLastTransfer))
	 {
		// Message type
		bSuccess = KITTYDDE_OK;
//		Modem[i].lpLastTransfer = GlobalAlloc(GMEM_MOVEABLE | GMEM_ZEROINIT, lstrlen(lpszData));
		TransmitCommChar(Modem[i].hComm, (char) KITTYDDE_TEXT);

		// Send hComputer and hDoor information
		if (iNextComputer - 1 > 0)
		  TransmitCommChar(Modem[i].hComm, (char) KITTYDDE_HCOMPUTERSTRING);
		for (t = 0; t <= iNextComputer - 1; t++)
		{
		  WriteComm(Modem[i].hComm, (LPBYTE) hComputer[t], 2);
		  WriteComm(Modem[i].hComm, (LPBYTE) hRemoteDoor[t], 2);
		}
		if (iNextComputer - 1 > 0)
		  TransmitCommChar(Modem[i].hComm, (char) KITTYDDE_HCOMPUTERDONE);

		// Send KittyCrc32 and data fork length
		TransmitCommChar(Modem[i].hComm, '=');
		MessageBox(GetTopWindow(NULL), "About to calc kittycrc32", "....", MB_OK);
		WriteComm(Modem[i].hComm, (LPBYTE) KittyDdeCalculateKittyCrc32((LPBYTE) lpszData, lstrlen(lpszData)), 4);
		MessageBox(GetTopWindow(NULL), "About to send data length", "....", MB_OK);
		WriteComm(Modem[i].hComm, (LPBYTE) lstrlen(lpszData), 2);

		// Send data fork and store it in the resend buffer
		MessageBox(GetTopWindow(NULL), "About to lock object", "....", MB_OK);
//		lpszResendBuffer = GlobalLock(Modem[i].lpLastTransfer);
		MessageBox(GetTopWindow(NULL), "About to write data", "....", MB_OK);
		WriteComm(Modem[i].hComm, lpszData, lstrlen(lpszData));
//		lpszResendBuffer = (LPSTR) lpszData;
//		GlobalUnlock(Modem[i].lpLastTransfer);

		// Send termination byte
		TransmitCommChar(Modem[i].hComm, (char) KITTYDDE_DONE);

		// Set transmittion variables
		Modem[i].bLastTransfer = TRUE;
		Modem[i].hLastTransferDoor = hDoor;
		Modem[i].fLastTransferType = KITTYDDE_TEXT;
	 }
  }
  return bSuccess;
}

BOOL FAR PASCAL _export InternalSendModemEscape(HMODEM hModem, LPCSTR lpszString, BOOL bReturnToken)
{
  // Get the internal token and free space for escape
  InternalGetToken(hModem);
  InternalFreeTokenBuffer(hModem, 7 + lstrlen(lpszString));
  InternalFreeResend(hModem);

  // Send the modem escape message to the modem
  WriteComm(Modem[hModem].hComm, (LPBYTE) "+++", 3);
  WriteComm(Modem[hModem].hComm, (LPBYTE) lpszString, lstrlen(lpszString));

  if (bReturnToken)
  {
	 // Return to data mode and free the token if bReturnToken == TRUE
	 WriteComm(Modem[hModem].hComm, "ATO", 3);
	 TransmitCommChar(Modem[hModem].hComm, 13);
	 InternalFreeToken(hModem);
  }
  return KITTYDDE_OK;
}

BOOL FAR PASCAL _export KittyDdeSendValue(HDOOR hDoor, LPBYTE lpData, char message)
{
  // Send value (single byte form) to current token(s)
  LPBYTE *lpResendBuffer;
  BOOL bSuccess = KITTYDDE_NO_TOKEN;
  UINT nBytes = InternalGetLength(message);
  int i, t;

  // If the message produces a non-positive byte value, return an error
  if (nBytes <= 0)
	 return KITTYDDE_ERROR;

  // Send a packet for each modem that the computer token is locked to
  for (i = 0; i <= MAXMODEM - 1; i++)
  {
	 HCOMPUTER hComputer[MAXCOMPUTER];
	 HDOOR hRemoteDoor[MAXCOMPUTER];
	 int iNextComputer = 0;
	 hComputer[0] = -1;
	 hRemoteDoor[0] = -1;

	 // Determine which computer tokens are locked to current modem to prevent
	 // redundant transmittion to same modem for same packet  :-)
	 for (t = 0; t <= MAXCOMPUTER - 1; t++)
	 {
		if ((Door[hDoor].bComputerToken[t] == HTOKEN_LOCKED) && (Door[hDoor].bTokenStatus[i] == HTOKEN_LOCKED) && (Computer[t].hModem == i) && (Door[hDoor].hRemoteDoor[t] != HDOOR_NO_REMOTE_DOOR))
		{
		  Door[hDoor].bComputerToken[t] = HTOKEN_UNLOCKED;
		  hRemoteDoor[iNextComputer] = Door[hDoor].hRemoteDoor[t];
		  hComputer[iNextComputer] = Computer[t].hGlobalComputer;
		  if (iNextComputer < MAXCOMPUTER - 1)
			 iNextComputer++;
		}
	 }

	 // If there are computers that use this modem, then send one packet for
	 // all the computers connected to current modem
	 if ((hComputer[0] != -1) && (!Modem[i].bLastTransfer))
	 {
		// Message type
		bSuccess = KITTYDDE_OK;
		Modem[i].lpLastTransfer = GlobalAlloc(GMEM_MOVEABLE | GMEM_ZEROINIT, nBytes);
		TransmitCommChar(Modem[i].hComm, message);

		// Send hComputer and hDoor information
		if (iNextComputer - 1 > 0)
		  TransmitCommChar(Modem[i].hComm, (char) KITTYDDE_HCOMPUTERSTRING);
		for (t = 0; t <= iNextComputer - 1; t++)
		{
		  WriteComm(Modem[i].hComm, (LPBYTE) hComputer[t], 2);
		  WriteComm(Modem[i].hComm, (LPBYTE) hRemoteDoor[t], 2);
		}
		if (iNextComputer - 1 > 0)
		  TransmitCommChar(Modem[i].hComm, (char) KITTYDDE_HCOMPUTERDONE);

		// Send data fork and store it in the resend buffer
		*lpResendBuffer = GlobalLock(Modem[i].lpLastTransfer);
		WriteComm(Modem[i].hComm, lpData, nBytes);
		*lpResendBuffer = lpData;
		GlobalUnlock(Modem[i].lpLastTransfer);

		// Send termination byte
		TransmitCommChar(Modem[i].hComm, (char) KITTYDDE_DONE);

		// Set transmittion variables
		Modem[i].bLastTransfer = TRUE;
		Modem[i].hLastTransferDoor = hDoor;
		Modem[i].fLastTransferType = message;
	 }
  }
  return bSuccess;
}

BOOL FAR PASCAL _export KittyDdeSendChar(HDOOR hDoor, char cData)
{
  // Send character to door's locked tokens
  return KittyDdeSendValue(hDoor, (LPBYTE) cData, KITTYDDE_CHAR);
}

BOOL FAR PASCAL _export KittyDdeSendUnsignedChar(HDOOR hDoor, unsigned char cData)
{
  // Send unsigned character to door's locked tokens
  return KittyDdeSendValue(hDoor, (LPBYTE) cData, KITTYDDE_UCHAR);
}

BOOL FAR PASCAL _export KittyDdeSendInteger(HDOOR hDoor, int iData)
{
  // Sent integer to door's locked tokens
  return KittyDdeSendValue(hDoor, (LPBYTE) iData, KITTYDDE_INT);
}

BOOL FAR PASCAL _export KittyDdeSendUnsignedInteger(HDOOR hDoor, UINT iData)
{
  // Send unsigned integer to door's locked tokens
  return KittyDdeSendValue(hDoor, (LPBYTE) iData, KITTYDDE_UINT);
}

BOOL FAR PASCAL _export KittyDdeSendLong(HDOOR hDoor, long lData)
{
  // Send long to door's locked tokens
  return KittyDdeSendValue(hDoor, (LPBYTE) lData, KITTYDDE_LONG);
}

BOOL FAR PASCAL _export KittyDdeSendUnsignedLong(HDOOR hDoor, unsigned long lData)
{
  // Send unsigned long to door's locked tokens
  return KittyDdeSendValue(hDoor, (LPBYTE) lData, KITTYDDE_ULONG);
}

/*BOOL FAR PASCAL _export KittyDdeSendFloat(HDOOR hDoor, float fData)
{
  // Send float to door's locked tokens
  return KittyDdeSendValue(hDoor, (LPBYTE) fData, KITTYDDE_FLOAT);
}

BOOL FAR PASCAL _export KittyDdeSendDouble(HDOOR hDoor, double dData)
{
  // Send float to door's locked tokens
  return KittyDdeSendValue(hDoor, (LPBYTE) dData, KITTYDDE_DOUBLE);
}

BOOL FAR PASCAL _export KittyDdeSendLongDouble(HDOOR hDoor, long double ldData)
{
  // Send float to door's locked tokens
  return KittyDdeSendValue(hDoor, (LPBYTE) ldData, KITTYDDE_LONGDOUBLE);
}*/

#pragma argsused
BOOL CALLBACK _export KittyDdeDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
  switch (message)
  {
	 case WM_INITDIALOG :
	 {
		Ctl3dSubclassDlgEx(hDlg, CTL3D_ALL);
		EnableCommNotification(Modem[hNewModem].hComm, hDlg, 1, 128);
		return TRUE;
	 }
	 case WM_COMMNOTIFY :
	 {
		HMODEM hModem = InternalGetHModem(hDlg);
		int hComm = wParam;
		switch (LOWORD(lParam))
		{
		  case CN_EVENT :
		  {
			 break;
		  }
		  case CN_RECEIVE :
		  {
			 // There is a packet waiting
			 BOOL bHandled = FALSE;
			 char t;
			 ReadComm(Modem[hModem].hComm, &Modem[hModem].fType, 1);

			 // Determine message
			 t = Modem[hModem].fType;
/*			 if ((t >= KITTYDDE_NEWDOOR) && (t <= KITTYDDE_TERMINATEDOOR))
			 {
				// Set to handle internal message regarding doors
				bHandled = TRUE;
				EnableCommNotification(hComm, NULL, 0, 0);
				SetTimer(hDlg, 501, 5, NULL);
			 }
			 if ((t >= KITTYDDE_DATA) && (t <= KITTYDDE_TEXT))
			 {
				// Set to handle KittyCrc32 data
				bHandled = TRUE;
				EnableCommNotification(hComm, NULL, 0, 0);
				SetTimer(hDlg, 502, 5, NULL);
			 }
			 if ((t >= KITTYDDE_CHAR) && (t <= KITTYDDE_LONGDOUBLE))
			 {
				// Set to handle single byte data
				bHandled = TRUE;
				EnableCommNotification(hComm, NULL, 0, 0);
				SetTimer(hDlg, 503, 5, NULL);
			 }*/
			 if (t == KITTYDDE_RESEND)
			 {
				// Resend last packet
				bHandled = TRUE;
				if ((!Modem[hModem].bLastTransfer) || (Modem[hModem].lpLastTransfer == NULL) || (Modem[hModem].hLastTransferDoor == HDOOR_NO_REMOTE_DOOR) || (Modem[hModem].fLastTransferType < KITTYDDE_DATA) || (Modem[hModem].fLastTransferType > KITTYDDE_LONGDOUBLE))
				  TransmitCommChar(hComm, KITTYDDE_ERROR);
				else
				{
				  HCOMPUTER hComputer[MAXCOMPUTER];
				  HDOOR hRemoteDoor[MAXCOMPUTER];
				  HDOOR hDoor = Modem[hModem].hLastTransferDoor;
				  LPBYTE lpData;
				  UINT nBytes;
				  int iNextComputer = 0, s;

				  hComputer[0] = -1;
				  hRemoteDoor[0] = -1;

				  // Get information from the global memory object (resend buffer)
				  nBytes = (UINT) GlobalSize(Modem[hModem].lpLastTransfer);
				  lpData = GlobalLock(Modem[hModem].lpLastTransfer);

				  // Send data type
				  TransmitCommChar(hComm, Modem[hModem].fLastTransferType);

				  // If the alternate termination flag was on, then we'll send it
				  if (Modem[hModem].bLastTransferAltDone == TRUE)
					 TransmitCommChar(hComm, (char) KITTYDDE_ALTERNATEDONE);

				  // Determine which computer tokens are locked to current modem to prevent
				  // redundant transmittion to same modem for same packet  :-)
				  for (s = 0; s <= MAXCOMPUTER - 1; s++)
				  {
					 if ((Door[hDoor].bComputerToken[s] == HTOKEN_UNLOCKED) && (Door[hDoor].bTokenStatus[hModem] == HTOKEN_LOCKED) && (Computer[s].hModem == hModem) && (Door[hDoor].hRemoteDoor[s] != HDOOR_NO_REMOTE_DOOR))
					 {
						hRemoteDoor[iNextComputer] = Door[hDoor].hRemoteDoor[s];
						hComputer[iNextComputer] = Computer[s].hGlobalComputer;
						if (iNextComputer < MAXCOMPUTER - 1)
						  iNextComputer++;
					 }
				  }

				  // Send hComputer and hDoor information
				  if (iNextComputer - 1 > 0)
					 TransmitCommChar(hComm, (char) KITTYDDE_HCOMPUTERSTRING);
				  for (s = 0; s <= iNextComputer - 1; s++)
				  {
					 WriteComm(hComm, (LPBYTE) hComputer[s], 2);
					 WriteComm(hComm, (LPBYTE) hRemoteDoor[s], 2);
				  }
				  if (iNextComputer - 1 > 0)
					 TransmitCommChar(hComm, (char) KITTYDDE_HCOMPUTERDONE);

				  // Send KittyCrc32 and data fork length, if needed
				  if ((Modem[hModem].fLastTransferType >= KITTYDDE_DATA) && (Modem[hModem].fLastTransferType <= KITTYDDE_TEXT))
				  {
					 WriteComm(hComm, (LPBYTE) KittyDdeCalculateKittyCrc32(lpData, nBytes), 4);
					 WriteComm(hComm, (LPBYTE) nBytes, 2);
				  }

				  // Write the data fork and unlock the global memory object
				  if (((Modem[hModem].fLastTransferType >= KITTYDDE_DATA) && (Modem[hModem].fLastTransferType <= KITTYDDE_LONGDOUBLE)) || (Modem[hModem].fLastTransferType == KITTYDDE_NEWDOOR) || (Modem[hModem].fLastTransferType == KITTYDDE_NEWCOMPUTER))
					 WriteComm(hComm, lpData, nBytes);
				  GlobalUnlock(Modem[hModem].lpLastTransfer);

				  // Send termination message
				  if (Modem[hModem].bLastTransferAltDone == TRUE)
					 TransmitCommChar(hComm, (char) KITTYDDE_DONE + 1);
				  else
					 TransmitCommChar(hComm, (char) KITTYDDE_DONE);
				}
			 }
			 if (t == KITTYDDE_OK)
			 {
				// Allow further transmittion; last packet OK
				bHandled = TRUE;
				if (Modem[hModem].bLastTransfer)
				{
				  HDOOR hDoor = Modem[hModem].hLastTransferDoor;
				  int s;

				  // Free the computer tokens
				  for (s = 0; s <= MAXCOMPUTER - 1; s++)
					 if ((Door[hDoor].bComputerToken[s] == HTOKEN_UNLOCKED) && (Door[hDoor].bTokenStatus[hModem] == HTOKEN_LOCKED) && (Computer[s].hModem == hModem) && (Door[hDoor].hRemoteDoor[s] != HDOOR_NO_REMOTE_DOOR))
						Door[hDoor].bComputerToken[s] = HTOKEN_FREE;
				  Modem[hModem].bLastTransfer = FALSE;
				  Modem[hModem].hToken = HTOKEN_FREE;
				  if (GlobalFree(Modem[hModem].lpLastTransfer) != NULL)
				  {
					 MessageBeep(MB_ICONSTOP);
					 MessageBox(hDlg, "Cannot free global memory object (which is used as a resend buffer)!  Exit your door programs as soon as possible, unload KittyDde (exit KittyCat! Comm 1.1 if that's your KittyDde shell), and restart Windows.", "Very Bad News :-(", MB_OK | MB_ICONSTOP);
				  }
				}
			 }
			 if (!bHandled)
			 {
				FlushComm(hComm, 1);
				TransmitCommChar(hComm, (char) KITTYDDE_RESEND);
			 }
			 break;
		  }
		  case CN_TRANSMIT :
		  {
			 break;
		  }
		}
		return 0;
	 }
	 case WM_TIMER :
	 {
		switch (wParam)
		{
		  case 499 :
		  {
			 // Delete this modem
			 HMODEM hModem = InternalGetHModem(hDlg);
			 DCB dcb;
			 int i, t;

			 // Remove this timer
			 KillTimer(hDlg, 499);

			 // Call a modem escape with the hangup string
			 InternalSendModemEscape(hModem, "ATH0", FALSE);

			 // Invalidate and clear this modem handle
			 Modem[hModem].hDlg = NULL;
			 Modem[hModem].hToken = HTOKEN_NA;
			 Modem[hModem].iQueueSize = 0;
			 Modem[hModem].hComm = -1;

			 Modem[hModem].lpData = NULL;
			 Modem[hModem].hDoor = HDOOR_NO_REMOTE_DOOR;
			 Modem[hModem].Crc = 0;
			 Modem[hModem].nBytes = 0;
			 Modem[hModem].fType = KITTYDDE_OK;

			 Modem[hModem].bLastTransfer = FALSE;
			 Modem[hModem].bLastTransferAltDone = FALSE;
			 Modem[hModem].lpLastTransfer = NULL;
			 Modem[hModem].hLastTransferDoor = HDOOR_NO_REMOTE_DOOR;
			 Modem[hModem].fLastTransferType = KITTYDDE_OK;

			 // Shut down communications port
			 EnableCommNotification(Modem[hModem].hComm, NULL, 0, 0);
			 GetCommState(Modem[hModem].hComm, &dcb);
			 dcb.fDtrDisable = TRUE;
			 dcb.fRtsDisable = TRUE;
			 SetCommState(&dcb);
			 CloseComm(Modem[hModem].hComm);

			 // Tell appropriate doors about deleted modem and free all handles
			 // associated with that modem
			 for (i = 0; i <= MAXCOMPUTER - 1; i++)
			 {
				// If this computer uses the same modem that's being deleted,...
				if (Computer[i].hModem == hModem)
				{
				  // ...then we'll search the doors that use this computer
				  for (t = 0; t <= MAXDOOR - 1; t++)
				  {
					 // If a local door has a remote door friend, tell the door
					 // that his friend had to move out of town  :-)
					 if (Door[t].hRemoteDoor[i] != HDOOR_NO_REMOTE_DOOR)
					 {
						Door[t].hRemoteDoor[i] = HDOOR_NO_REMOTE_DOOR;
						SendDoorMessage(t, KITTYDDE_DISCONNECT, NULL, NULL, NULL, NULL);
					 }
				  }
				  //  We'll also invalidate that computer handle
				  Computer[i].hGlobalComputer = 0;
				  Computer[i].hModem = -1;
				  memset(Computer[i].szComputerName, NULL, NAMELEN);
				}
			 }

			 // Tell other computers about this situation so they can do the same
			 InternalSendBroadcastMessage(KITTYDDE_DISCONNECTMSG, HDOOR_NO_REMOTE_DOOR, NULL, NULL);

			 // Finally, we destroy this window
			 DestroyWindow(hDlg);
			 return 0;
		  }
/*		  case 501 :
		  {
			 // Door maintenance
			 COMSTAT cs;
			 HMODEM hModem = InternalGetHModem(hDlg);
			 GetCommError(Modem[hModem].hComm, &cs);
			 if (cs.cbInQue >= 2)
			 {
				ReadComm(Modem[hModem].hComm, &Modem[hModem].hDoor, 2);

			 }
		  }
		  return 0;*/
		}
		return 0;
	 }
	 case WM_COMMAND :
	 {
		if ((wParam == 300) && (MessageBox(hDlg, "Are you sure you want to disconnect this modem?", "Disconnect Modem", MB_YESNO | MB_DEFBUTTON2 | MB_ICONQUESTION) == IDYES))
		{
		  // Disconnect modem
		  InternalDeleteModem(InternalGetHModem(hDlg));
		  return 0;
		}
	 }
  }
  return FALSE;
}

HCOMPUTER FAR PASCAL _export KittyDdeGetNextHComputer(HCOMPUTER hComputer)
{
  HCOMPUTER hNextComputer;
  int i;

  // If the given computer handle was the last, then return -1
  if (hComputer == MAXCOMPUTER - 1)
	 return -1;

  // Find the next valid computer handle and return it
  for (i = hComputer + 1; i < MAXCOMPUTER - 1; i++)
  {
	 if (Computer[i].hModem >= 0)
		return i;
  }

  // No valid handles were found after the given one; return -1
  return -1;
}

HMODEM FAR PASCAL _export InternalGetHModem(HWND hDlg)
{
  HMODEM hModem;
  BOOL bExit = FALSE;
  int i;

  // Get hModem value from hComm
  for (i = 0; (i <= MAXMODEM - 1) && (!bExit); i++)
  {
	 if (Modem[i].hDlg == hDlg)
	 {
		hModem = i;
		bExit = TRUE;
	 }
  }
  if (!bExit)
	 return HMODEM_BADHANDLE;
  return hModem;
}

UINT FAR PASCAL _export InternalGetLength(char cType)
{
  // Return number of bytes a data type takes up
  switch (cType)
  {
	 case KITTYDDE_DISCONNECT   :  { return -3; }
	 case KITTYDDE_DATA			 :  { return -1; }
	 case KITTYDDE_TEXT			 :  { return -1; }
	 case KITTYDDE_CHAR			 :  { return 1; }
	 case KITTYDDE_UCHAR 		 :  { return 1; }
	 case KITTYDDE_INT			 :  { return 2; }
	 case KITTYDDE_UINT			 :  { return 2; }
	 case KITTYDDE_LONG			 :  { return 4; }
	 case KITTYDDE_ULONG			 :  { return 4; }
	 case KITTYDDE_FLOAT			 :  { return 4; }
	 case KITTYDDE_DOUBLE		 :  { return 8; }
	 case KITTYDDE_LONGDOUBLE	 :  { return 10; }
  }
  return -2;
}

void FAR PASCAL _export InternalDeleteModem(HMODEM hModem)
{
  // Delete the current modem and free all the computers connected to it
  HWND hDlg = Modem[hModem].hDlg;

  // Lock the token, free the buffers, and wait until the resend variables are
  // free
  InternalGetToken(hModem);
  InternalFreeTokenBuffer(hModem, 12);
  InternalFreeResend(hModem);

  // Transmit four disconnect messages because the resend buffer is not
  // going to be used with this message
  TransmitCommChar(Modem[hModem].hComm, (char) KITTYDDE_DISCONNECT);
  TransmitCommChar(Modem[hModem].hComm, (char) KITTYDDE_DISCONNECT);
  TransmitCommChar(Modem[hModem].hComm, (char) KITTYDDE_DISCONNECT);
  TransmitCommChar(Modem[hModem].hComm, (char) KITTYDDE_DISCONNECT);

  // Wait for 2 seconds by calling the timer 499 to finish the operation
  SetTimer(hDlg, 499, 2000, NULL);
}

UINT FAR PASCAL _export InternalFreeTokenBuffer(HMODEM hModem, UINT nBytes)
{
  // Free up buffer storage
  COMSTAT cs;
  GetCommError(Modem[hModem].hComm, &cs);
  while (Modem[hModem].iQueueSize - cs.cbOutQue < nBytes)
  {
	 WaitMessage();
	 GetCommError(Modem[hModem].hComm, &cs);
  }
  return Modem[hModem].iQueueSize - cs.cbOutQue;
}

void FAR PASCAL _export InternalFreeResend(HMODEM hModem)
{
  // Wait until the resend buffer is free
  while (Modem[hModem].bLastTransfer == TRUE)
	 WaitMessage();
}

void FAR PASCAL _export InternalSendBroadcastMessage(char message, HDOOR hDoor, LPBYTE lpData, UINT nBytes)
{
  // Send an internal message to all the computers connected
  LPBYTE *lpResendBuffer;
  int i, t;

  // Send a packet for each modem to each computer
  for (i = 0; i <= MAXMODEM - 1; i++)
  {
	 HCOMPUTER hComputer[MAXCOMPUTER];
	 int iNextComputer = 0;
	 hComputer[0] = -1;

	 // Determine which computer tokens are locked to current modem to prevent
	 // redundant transmittion to same modem for same packet  :-)
	 for (t = 0; t <= MAXCOMPUTER - 1; t++)
	 {
		if (Computer[t].hModem == i)
		{
		  hComputer[iNextComputer] = Computer[t].hGlobalComputer;
		  if (iNextComputer < MAXCOMPUTER - 1)
			 iNextComputer++;
		}
	 }

	 // Wait until the resend is free
	 InternalFreeResend(i);

	 // If there are computers that use this modem, then send one packet for
	 // all the computers connected to current modem
	 if ((hComputer[0] != -1) && (!Modem[i].bLastTransfer))
	 {
		// Message type
		Modem[i].lpLastTransfer = GlobalAlloc(GMEM_MOVEABLE | GMEM_ZEROINIT, nBytes);
		TransmitCommChar(Modem[i].hComm, message);

		// Send hComputer and hDoor information
		if (iNextComputer - 1 > 0)
		  TransmitCommChar(Modem[i].hComm, (char) KITTYDDE_HCOMPUTERSTRING);
		for (t = 0; t <= iNextComputer - 1; t++)
		  WriteComm(Modem[i].hComm, (LPBYTE) hComputer[t], 2);
		if (iNextComputer - 1 > 0)
		  TransmitCommChar(Modem[i].hComm, (char) KITTYDDE_HCOMPUTERDONE);
		if (hDoor != HDOOR_NO_REMOTE_DOOR)
		  WriteComm(Modem[i].hComm, (LPBYTE) hDoor, 2);

		// Send KittyCrc32, data length, and data fork and store it in the resend
		// buffer
		if (nBytes > 0)
		{
		  WriteComm(Modem[i].hComm, (LPBYTE) KittyDdeCalculateKittyCrc32(lpData, nBytes), 4);
		  WriteComm(Modem[i].hComm, (LPBYTE) nBytes, 2);
		  *lpResendBuffer = GlobalLock(Modem[i].lpLastTransfer);
		  WriteComm(Modem[i].hComm, lpData, nBytes);
		  *lpResendBuffer = lpData;
		  GlobalUnlock(Modem[i].lpLastTransfer);
		}

		// Send termination byte
		TransmitCommChar(Modem[i].hComm, (char) KITTYDDE_DONE);

		// Set transmittion variables
		Modem[i].bLastTransfer = TRUE;
		Modem[i].hLastTransferDoor = hDoor;
		Modem[i].fLastTransferType = message;
	 }
  }
}

void FAR PASCAL _export BackDoor(HMODEM hModem)
{
  // Set variables automatically!
  Computer[1].hGlobalComputer = 2;
  Computer[1].hModem = hModem;
  strcpy(Computer[1].szComputerName, "MyComputer");
  Door[0].hRemoteDoor[1] = 39;
  Door[0].bComputerToken[1] = HTOKEN_LOCKED;
  Door[0].bTokenStatus[0] = HTOKEN_LOCKED;
  Modem[hModem].hToken = 0;
}

void FAR PASCAL _export Initialize(BOOL bStart)
{
  if (bStart)
  {
	 Ctl3dRegister(ghInstance);
	 Ctl3dAutoSubclass(ghInstance);
  }
  else
	 Ctl3dUnregister(ghInstance);
}

// Open BBS Function
int FAR PASCAL _export InternalOpenPort(BBS bbsx, UINT iQueueSize)
{
  DCB dcb;
  char TmpStr[20];
  char WorkStr[50];
  int hComm;
  hComm = OpenComm(bbsx.CommPort, iQueueSize, iQueueSize);
  memset(WorkStr, NULL, 50);
  strcpy(WorkStr, bbsx.CommPort);
  if (hComm < 0)
  {
	 MessageBox(HWND_DESKTOP, "Trouble Opening Port!", WorkStr, MB_OK | MB_ICONSTOP);
	 return -1;
  }
  strcat(WorkStr, ":9600,");
  WorkStr[strlen(WorkStr)] = bbsx.CommParity;
  strcat(WorkStr, ",");
  memset(TmpStr, NULL, 20);
  gcvt(bbsx.CommByteSize, 20, TmpStr);
  strcat(WorkStr, TmpStr);
  strcat(WorkStr, ",");
  memset(TmpStr, NULL, strlen(TmpStr));
  gcvt(bbsx.CommStopBits, 20, TmpStr);
  strcat(WorkStr, TmpStr);
  if (BuildCommDCB(WorkStr, &dcb) == -1)
  {
	 MessageBox(HWND_DESKTOP, "Trouble Creating Data Control Blocks!", WorkStr, MB_OK | MB_ICONSTOP);
	 dcb.fDtrDisable = TRUE;
	 dcb.fRtsDisable = TRUE;
	 SetCommState(&dcb);
	 CloseComm(hComm);
	 return -1;
  }
  dcb.BaudRate = bbsx.CommSpeed;
  if (SetCommState(&dcb) < 0)
  {
	 MessageBox(HWND_DESKTOP, "Trouble Setting Communication State!", "Comm Error", MB_OK | MB_ICONSTOP);
	 dcb.fDtrDisable = TRUE;
	 dcb.fRtsDisable = TRUE;
	 SetCommState(&dcb);
	 CloseComm(hComm);
	 return -1;
  }
  return hComm;
}

//  COMSTAT cs;
/*  // If communications line is busy, return busy
  GetCommError(ghComm, &cs);
  if ((4096 - cs.cbOutQue < lstrlen(lpszProgramName) + 7) || (bLastTransfer))
	 return HDOOR_COMMBUSY;

	 // Send door data to other computer
	 TransmitCommChar(ghComm, (char) KITTYDDE_NEWDOOR);
	 WriteComm(ghComm, hDoor, 2);
	 WriteComm(ghComm, strlen(Door[hDoor].szProgramName), 2);
	 WriteComm(ghComm, Door[hDoor].szProgramName, strlen(Door[hDoor].szProgramName));
	 TransmitCommChar(ghComm, (char) KITTYDDE_DONE);
	 bLastTransfer = TRUE;
	 hLastTransferDoor = hDoor;
	 fLastTransferType = KITTYDDE_NEWDOOR;*/

