#if !defined(__fledtapp_h)              // Sentry, use file only if it's not already included.
#define __fledtapp_h

/*  Project FileEdit
    KittyCat! Systems
    Copyright  1995 Steven Lawrance. All Rights Reserved.

    SUBSYSTEM:    fileedit.exe Application
    FILE:         fledtapp.h
    AUTHOR:       Steven Lawrance


    OVERVIEW
    ========
    Class definition for FileEditApp (TApplication).      
*/


#include <owl\owlpch.h>
#pragma hdrstop

#include <ocf\automacr.h>

#include <classlib\bags.h>


#include "apxprint.h"
#include "apxprev.h"

#include "fledtapp.rh"            // Definition of all resources.


// TFileDrop class Maintains information about a dropped file, its name, where it was dropped,
// and whether or not it was in the client area
class TFileDrop {
public:
    operator == (const TFileDrop& other) const {return this == &other;}

    char*   FileName;
    TPoint  Point;
    bool    InClientArea;

    TFileDrop (char*, TPoint&, bool, TModule*);
    ~TFileDrop ();

    const char* WhoAmI ();
private:
    //
    // hidden to prevent accidental copying or assignment
    //
    TFileDrop (const TFileDrop&);
    TFileDrop & operator = (const TFileDrop&);
};

typedef TIBagAsVector<TFileDrop> TFileList;
typedef TIBagAsVectorIterator<TFileDrop> TFileListIter;


//
// FrameWindow must be derived to override Paint for Preview and Print.
//
//{{TOleFrame = SDIDecFrame}}
class SDIDecFrame : public TOleFrame {
public:
    SDIDecFrame (TWindow *parent, const char far *title, TWindow *clientWnd, bool trackMenuSelection = false, TModule *module = 0);
    ~SDIDecFrame ();
};    //{{SDIDecFrame}}


//{{TApplication = FileEditApp}}
class FileEditApp : public TApplication, public TOcModule {
private:

private:
    void SetupSpeedBar (TOleFrame *frame);
    void AddFile (const char *fileName);

public:
    FileEditApp ();
    virtual ~FileEditApp ();

    void CreateGadgets (TControlBar *cb, bool server = false);

    // Public data members used by the print menu commands and Paint routine in MDIChild.
    TPrinter        *Printer;                           // Printer support.
    int             Printing;                           // Printing in progress.

//{{FileEditAppVIRTUAL_BEGIN}}
public:
    virtual void InitMainWindow ();
    virtual void InitInstance ();
//{{FileEditAppVIRTUAL_END}}

//{{FileEditAppRSP_TBL_BEGIN}}
protected:
    void EvNewView (TView& view);
    void EvCloseView (TView& view);
    void CmHelpAbout ();
    void CmFilePrint ();
    void CmFilePrintSetup ();
    void CmFilePrintPreview ();
    void CmPrintEnable (TCommandEnabler &tce);
    void EvDropFiles (TDropInfo drop);
    void EvWinIniChange (char far* section);
//{{FileEditAppRSP_TBL_END}}

DECLARE_AUTOAGGREGATE(FileEditApp)
AUTODETACH
//{{FileEditAppAUTO_TBL_BEGIN}}
//{{FileEditAppAUTO_TBL_END}}

DECLARE_RESPONSE_TABLE(FileEditApp);
};    //{{FileEditApp}}


#endif                                      // __fledtapp_h sentry.
