// KittyCat! Comm 1.0 for Windows 3.1
// by Steven Lawrance
// ---Resource File---

#include <windowsx.h>
#include <windows.h>
#include <string.h>
#include "Structs.h"
#include "Userintr.h"

// Global Variables
HINSTANCE dllInstance;
HPALETTE hNewPalette;
PICS pics;

/***********************************/

// DLL Routines
HINSTANCE FAR PASCAL _export Resource_GetDLLInstance(void);

// Window Picture Routines
void FAR PASCAL _export RefreshPicPal(HWND hWnd, HPALETTE hNewPalette);
void FAR PASCAL _export RefreshPicture(HWND hWnd, HBITMAP BitmapHandle, HPALETTE hNewPalette, int wx, int wy, int bx, int by, int bw, int bh);
HBITMAP FAR PASCAL _export GetPic256(HWND hWnd, char PicName[20]);
BOOL FAR PASCAL _export DeletePal(HPALETTE hNewPalette);
HPALETTE FAR PASCAL _export GetPicPal(void);

// Button Routines
void FAR PASCAL _export Draw16(LPSTR Bitmap, DRAWITEMSTRUCT FAR *ds);
void FAR PASCAL _export DrawButtonState(HWND hDlg, LPDRAWITEMSTRUCT lpdis, char ButtonName[30]);
LPMEASUREITEMSTRUCT FAR PASCAL _export InitButton(LPMEASUREITEMSTRUCT lpmis, int ID, int X, int Y);

// Dialog Picture Routine
void FAR PASCAL _export DialogPicture(HBITMAP BitmapHandle, HPALETTE hNewPalette, DRAWITEMSTRUCT FAR* dis);

// "About" Dialog Box Routines
void FAR PASCAL _export About(HWND hWnd);
BOOL CALLBACK _export AboutDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
void FAR PASCAL _export WMAboutDlgCommand_Handler(HWND hDlg, int id, HWND hwndCtl, UINT codeNotify);

/************************************/

// DLL Routines
#pragma argsused
int FAR PASCAL LibMain (HINSTANCE hInstance, WORD wDataSeg, WORD wHeapSize, LPSTR lpszCmdLine)
{
  if (wHeapSize > 0)
    UnlockData(0);
  dllInstance = hInstance;
  return 1;
}

HINSTANCE FAR PASCAL _export Resource_GetDLLInstance(void)
{
  return dllInstance;
}


// Window Picture Routines
void FAR PASCAL _export RefreshPicPal(HWND hWnd, HPALETTE hNewPalette)
{
  if (hNewPalette)
  {
    HPALETTE hOldPalette;
    HDC hDC = GetDC(hWnd);
    hOldPalette = SelectPalette(hDC,hNewPalette,FALSE);
    RealizePalette(hDC);
    SelectPalette(hDC,hOldPalette,FALSE);
    ReleaseDC(hWnd,hDC);
    DeleteObject(hOldPalette);
  }
}

void FAR PASCAL _export RefreshPicture(HWND hWnd, HBITMAP BitmapHandle, HPALETTE hNewPalette, int wx, int wy, int bx, int by, int bw, int bh)
{
  RECT rect;
  HBITMAP OldBitmapHandle;
  BITMAP BM;
  HDC hDC = GetDC(hWnd);
  HDC MemoryDC;
  HPALETTE hOldPal;
  GetClientRect(hWnd, &rect);
  GetObject(BitmapHandle, sizeof(BITMAP), &BM);
  if (wx == 0)
  {
    wx = rect.left;
  }
  if (wy == 0)
  {
    wy = rect.top;
  }
  if (bw == 0)
  {
    bw = BM.bmWidth;
  }
  if (bh == 0)
  {
    bh = BM.bmHeight;
  }
  if ( BitmapHandle )
  {
    MemoryDC = CreateCompatibleDC(hDC);
    OldBitmapHandle = (HBITMAP)SelectObject(MemoryDC, BitmapHandle);
    hOldPal = SelectPalette( hDC, hNewPalette, FALSE);
    SetBkColor(hDC, 0L);
    SetTextColor(hDC, 0xFFFFFFL);
    BitBlt(hDC, wx, wy, bw, bh, MemoryDC, bx, by, SRCCOPY);
    SelectPalette( hDC, hOldPal, FALSE);
    SelectObject(MemoryDC, OldBitmapHandle);
    DeleteDC(MemoryDC);
  }
  ValidateRect(hWnd,NULL);
  ReleaseDC(hWnd, hDC);
  ReleaseDC(hWnd, MemoryDC);
  DeleteObject(OldBitmapHandle);
  DeleteObject(hOldPal);
}

HBITMAP FAR PASCAL _export GetPic256(HWND hWnd, char PicName[20])
{
  HPALETTE hOldPalette;
  RGBQUAD FAR  *pRgb;
  LPLOGPALETTE lpDstPal;
  HGLOBAL hBitmap = NULL;
  LPBITMAPINFO pBits;
  short nColors, Count;
  HBITMAP BitmapHandle;
  hNewPalette     = NULL;
  hBitmap  = LoadResource(dllInstance, FindResource(dllInstance, (LPSTR)PicName, RT_BITMAP));
  if (!hBitmap)
  {
    MessageBeep(MB_ICONEXCLAMATION);
  }
  if ( hBitmap )
  {
    HDC hDC = GetDC(hWnd);
    pBits = (LPBITMAPINFO)GlobalLock(hBitmap);
    nColors = 1<<(pBits->bmiHeader.biBitCount);
    if ( nColors)
    {
      lpDstPal = (LPLOGPALETTE) GlobalLock( GlobalAlloc(
      GHND, sizeof(LOGPALETTE) + ( (nColors - 1) * sizeof(PALETTEENTRY) )));
      lpDstPal->palNumEntries = nColors;
      lpDstPal->palVersion    = 0x300;
      pRgb = pBits->bmiColors;
      for (Count = 0; Count <= nColors; Count++)
      {
        lpDstPal->palPalEntry[Count].peRed   = pRgb[Count].rgbRed;
	lpDstPal->palPalEntry[Count].peGreen = pRgb[Count].rgbGreen;
	lpDstPal->palPalEntry[Count].peBlue  = pRgb[Count].rgbBlue;
	lpDstPal->palPalEntry[Count].peFlags = 0;
      }
    }
    hNewPalette = CreatePalette(lpDstPal);
    GlobalFree( (HANDLE) GlobalHandle( HIWORD( lpDstPal) ) );
    hOldPalette = SelectPalette(hDC,hNewPalette,FALSE);
    RealizePalette(hDC);
    BitmapHandle = CreateDIBitmap(hDC, (LPBITMAPINFOHEADER)pBits, CBM_INIT, ((LPSTR) pBits + 40) + (nColors * sizeof(RGBQUAD)), pBits, DIB_RGB_COLORS);
    SelectPalette(hDC,hOldPalette,FALSE);
    FreeResource(hBitmap);
    ReleaseDC(hWnd,hDC);
    DeleteObject(hOldPalette);
  }
  return BitmapHandle;
}

HPALETTE FAR PASCAL _export GetPicPal(void)
{
  return hNewPalette;
}

BOOL FAR PASCAL _export DeletePal(HPALETTE hNewPalette)
{
  return DeleteObject(hNewPalette);
}


// Button Routines
void FAR PASCAL _export Draw16(LPSTR Bitmap, DRAWITEMSTRUCT FAR *ds)
{
  HBITMAP hBitmap;
  HBITMAP hOldBitmap;
  HDC MemDC = CreateCompatibleDC(ds->hDC);
  hBitmap = LoadBitmap(dllInstance, Bitmap);
  hOldBitmap = SelectObject(MemDC, hBitmap);
  BitBlt(ds->hDC, ds->rcItem.left, ds->rcItem.top, ds->rcItem.right, ds->rcItem.bottom, MemDC, 0, 0, SRCCOPY);
  SelectObject(MemDC, hOldBitmap);
  ReleaseDC(ds->hwndItem, MemDC);
  DeleteDC(MemDC);
  DeleteObject(hBitmap);
  DeleteObject(hOldBitmap);
}

void FAR PASCAL _export DrawButtonState(HWND hDlg, LPDRAWITEMSTRUCT lpdis, char ButtonName[30])
{
  DWORD ButtonState;
  ButtonState = SendDlgItemMessage(hDlg, lpdis->CtlID, BM_GETSTATE, NULL, NULL);
  if (ButtonState & 0x0008)
  {
    if (ButtonState & 0x0004)
    {
      strcat(ButtonName, "_DOWN");
      Draw16(ButtonName, lpdis);
    }
    else
    {
      strcat(ButtonName, "_HILITE");
      Draw16(ButtonName, lpdis);
    }
  }
  else
  {
    strcat(ButtonName, "_UP");
    Draw16(ButtonName, lpdis);
  }
}

LPMEASUREITEMSTRUCT FAR PASCAL _export InitButton(LPMEASUREITEMSTRUCT lpmis, int ID, int X, int Y)
{
  lpmis->CtlType = ODT_BUTTON;
  lpmis->CtlID = ID;
  lpmis->itemID = NULL;
  lpmis->itemWidth = X;
  lpmis->itemHeight = Y;
  lpmis->itemData = NULL;
  return lpmis;
}


// Dialog Picture Routine
void FAR PASCAL _export DialogPicture(HBITMAP BitmapHandle, HPALETTE hNewPalette, DRAWITEMSTRUCT FAR* dis)
{
  HBITMAP OldBitmapHandle;
  HDC MemoryDC;
  HPALETTE hOldPal;

  if ( BitmapHandle )
  {
    MemoryDC = CreateCompatibleDC(dis->hDC);
    OldBitmapHandle = (HBITMAP)SelectObject(MemoryDC, BitmapHandle);
    hOldPal = SelectPalette(dis->hDC, hNewPalette, FALSE);
    SetBkColor(dis->hDC, 0L);
    SetTextColor(dis->hDC, 0xFFFFFFL);
    BitBlt(dis->hDC, dis->rcItem.left, dis->rcItem.top, dis->rcItem.right, dis->rcItem.bottom, MemoryDC, 0, 0, SRCCOPY);
    SelectPalette(dis->hDC, hOldPal, FALSE);
    SelectObject(MemoryDC, OldBitmapHandle);
    ReleaseDC(dis->hwndItem, MemoryDC);
    DeleteDC(MemoryDC);
  }
  DeleteObject(OldBitmapHandle);
  DeleteObject(hOldPal);
}


// "About" Dialog Box Routines
void FAR PASCAL _export About(HWND hWnd)
{
  DLGPROC DlgProc;
  DlgProc = (DLGPROC)MakeProcInstance((FARPROC)AboutDlgProc, dllInstance);
  DialogBox(dllInstance, "ABOUT", hWnd, DlgProc);
  FreeProcInstance((FARPROC)DlgProc);
}

#pragma argsused
BOOL CALLBACK _export AboutDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
  switch(message)
  {
    case WM_INITDIALOG :
    {
      if (!pics.hBitmap1)
      {
        ChangeCursor(hDlg, (LPSTR) IDC_WAIT);
        pics.hBitmap1 = GetPic256(hDlg, "CATS_BMP");
        pics.hPalette1 = GetPicPal();
	SetMainWindowPics(pics);
	ChangeCursor(hDlg, (LPSTR) IDC_ARROW);
      }
      return TRUE;
    }
    case WM_SETFOCUS :
    {
      RefreshPicPal(hDlg, pics.hPalette1);
      return 0;
    }
    case WM_MEASUREITEM :
    {
      LPMEASUREITEMSTRUCT lpmis1;
      LPMEASUREITEMSTRUCT lpmis2;
      lpmis1 = (MEASUREITEMSTRUCT FAR*) lParam;
      lpmis2 = (MEASUREITEMSTRUCT FAR*) lParam;
      lpmis1 = InitButton(lpmis1, 300, 230, 190);
      lpmis2 = InitButton(lpmis2, 301, 222, 90);
      return TRUE;
    }
    case WM_DRAWITEM :
    {
      LPDRAWITEMSTRUCT lpdis;
      lpdis = (DRAWITEMSTRUCT FAR*) lParam;
      RefreshPicPal(hDlg, pics.hPalette1);
      if (lpdis->CtlID == 300)
      {
	if (lpdis->itemAction == ODA_DRAWENTIRE)
	  DialogPicture(pics.hBitmap1, pics.hPalette1, lpdis);
      }
      if (lpdis->CtlID == 301)
	Draw16("CATS_MEOW", lpdis);
    }
    case WM_COMMAND :
    {
      RefreshPicPal(hDlg, pics.hPalette1);
      return (BOOL)HANDLE_WM_COMMAND(hDlg, wParam, lParam, WMAboutDlgCommand_Handler);
    }
  }
  return FALSE;
}

#pragma argsused
void FAR PASCAL _export WMAboutDlgCommand_Handler(HWND hDlg, int id, HWND hwndCtl, UINT codeNotify)
{
  switch(id)
  {
    case 300 :
    {
      if (codeNotify == BN_CLICKED)
	WaveFile("play cats from 0");
      break;
    }
    case 2 :
    case 1 :
    {
      EndDialog(hDlg, 0);
      break;
    }
  }
}
