// KittyCat! Comm 1.0 for Windows 3.1
// by Steven Lawrance
// ---User Interface File---

#include <windowsx.h>
#include <windows.h>
#include <commdlg.h>
#include <string.h>
#include <mmsystem.h>
#include <stdlib.h>
#include "Resource.h"
#include "Config.h"
#include "Structs.h"
#include "Dialdir.h"
#include "Distrbtr.h"
#include "CatServe.h"

// Global Variables
HINSTANCE dllInstance, ResourceInstance;
char IniFile[144];
DWORD PlaySounds, AutoArrange;
FILESTRUCT fs;
PICS pics;
HMENU hMenu;
HWND hMDIClientWnd, ghWnd;

/***********************************/

// DLL Routines
HINSTANCE FAR PASCAL _export Userintr_GetDLLInstance(void);

// Main Window
MSG FAR PASCAL _export RunProgram(HINSTANCE hPrevInstance, LPSTR lpCmdParam, int nCmdShow);
LRESULT CALLBACK _export MainWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
void FAR PASCAL _export WMCommand_Handler(HWND hWnd, int id, HWND hwndCtl, UINT codeNotify);

// Program Functions
HWND FAR PASCAL _export GetGlobalWindow(void);
DWORD FAR PASCAL _export IsVersionRegistered(void);
void FAR PASCAL _export Sound(UINT AlertLevel);
void FAR PASCAL _export WaveFile(LPCSTR String);
void FAR PASCAL _export ChangeCursor(HWND hWnd, LPSTR hCursor);
RESULT FAR PASCAL _export LoadResultTable(RESULT Result);
void FAR PASCAL _export SaveResultTable(RESULT Result);
DATASTRUCT FAR PASCAL _export LoadDefaultStruct(DATASTRUCT DataStruct);
void FAR PASCAL _export SaveDefaultStruct(DATASTRUCT DataStruct);
void FAR PASCAL _export SetMainWindowPics(PICS Pics);
void CloseMDIChildren(void);
void CutBBS(HWND hWnd);
void CopyBBS(HWND hWnd, LPCSTR BBSName);
void PasteBBS(HWND hWnd);
int FAR PASCAL _export GetSoundVariable(void);
void FileDialogRunner(HWND hWnd, char WorkStr[144], char *DialogText, char *DefaultText, char *FileString, char *DefExt, DWORD MustExist, DWORD OverwritePrompt);

// Dialing Directory
HWND FAR PASCAL _export MakeNewChild(char *pName);
LRESULT CALLBACK _export MDIChildWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);

/************************************/

// DLL Routines
#pragma argsused
int FAR PASCAL LibMain (HINSTANCE hInstance, WORD wDataSeg, WORD wHeapSize, LPSTR lpszCmdLine)
{
  if (wHeapSize > 0)
    UnlockData(0);
  dllInstance = hInstance;
  return 1;
}

HINSTANCE FAR PASCAL _export Userintr_GetDLLInstance(void)
{
  return dllInstance;
}


// Main Window
MSG FAR PASCAL _export RunProgram(HINSTANCE hPrevInstance, LPSTR lpCmdParam, int nCmdShow)
{
  char        ProgName[] = "KittyCat! Systems";
  HWND        hWnd;
  HACCEL      hAccel;
  MSG 	      msg;

  ResourceInstance = Resource_GetDLLInstance();
  if (!hPrevInstance)
  {
    WNDCLASS    wndclass;
    wndclass.lpszClassName = ProgName;
    wndclass.lpfnWndProc   = (WNDPROC) MainWndProc;
    wndclass.cbClsExtra    = 0;
    wndclass.cbWndExtra    = 0;
    wndclass.hInstance     = dllInstance;
    wndclass.hIcon         = LoadIcon(ResourceInstance, "CATCOMM");
    wndclass.hCursor       = LoadCursor(NULL, IDC_ARROW);
    wndclass.hbrBackground = (HBRUSH) (COLOR_APPWORKSPACE + 1);
    wndclass.lpszMenuName  = "MAIN";
    wndclass.style         = CS_VREDRAW | CS_HREDRAW;
    RegisterClass(&wndclass);
    wndclass.style         = WS_MINIMIZE | WS_DLGFRAME;
    wndclass.lpfnWndProc   = (WNDPROC) MDIChildWndProc;
    wndclass.hIcon         = LoadIcon(ResourceInstance, "DIALMDI");
    wndclass.hCursor       = LoadCursor(NULL, IDC_ARROW);
    wndclass.hbrBackground = (HBRUSH) (COLOR_WINDOW + 1);
    wndclass.lpszMenuName  = NULL;
    wndclass.cbClsExtra    = 0;
    wndclass.cbWndExtra    = sizeof(LOCALHANDLE);
    wndclass.lpszClassName = "DialMDI";
    RegisterClass(&wndclass);
  }

  hAccel = LoadAccelerators(ResourceInstance, "ACCEL");
  hMenu = LoadMenu(ResourceInstance, "MAIN");
  if (OpenFile(lpCmdParam, NULL, OF_EXIST) == TRUE)
  {
    strcpy(IniFile, lpCmdParam);
  }
  else
  {
    strcpy(IniFile, "KittyCom.Ini");
  }
  hWnd = CreateWindow(ProgName,"KittyCat! Comm 1.0",
		      WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN,
                      CW_USEDEFAULT, CW_USEDEFAULT,          
                      CW_USEDEFAULT, CW_USEDEFAULT,          
                      NULL, NULL, dllInstance, NULL);

  SetMenu(hWnd, hMenu);
  ShowWindow(hWnd, nCmdShow);
  UpdateWindow(hWnd);
  LoadBBSEntriesIntoMDI(fs);

  while (GetMessage(&msg, NULL, 0, 0))
  {
    if (!TranslateMDISysAccel(hMDIClientWnd, &msg))
    {
      TranslateMessage(&msg);
      TranslateAccelerator(hWnd, hAccel, &msg);
      DispatchMessage(&msg);
    }
  }
  DestroyMenu(hMenu);
  UnregisterClass(ProgName, dllInstance);
  UnregisterClass("DialMDI", dllInstance);
  return msg;
}

LRESULT CALLBACK _export MainWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
  switch (message)
  {
    case WM_CREATE :
    {
      CLIENTCREATESTRUCT ccs;
      char WorkStr[5];
      GetPrivateProfileString("Setup", "Sounds", "on", WorkStr, 5, IniFile);
      if (strcmp(WorkStr, "on") == 0)
      {
	PlaySounds = TRUE;
	CheckMenuItem(hMenu, 100, MF_CHECKED);
      }
      if (strcmp(WorkStr, "off") == 0)
      {
	PlaySounds = FALSE;
	CheckMenuItem(hMenu, 100, MF_UNCHECKED);
      }
      GetPrivateProfileString("Setup", "AutoArrange", "on", WorkStr, 5, IniFile);
      if (strcmp(WorkStr, "on") == 0)
      {
	AutoArrange = TRUE;
	CheckMenuItem(hMenu, 401, MF_CHECKED);
      }
      if (strcmp(WorkStr, "off") == 0)
      {
	AutoArrange = FALSE;
	CheckMenuItem(hMenu, 401, MF_UNCHECKED);
      }
      memset(&fs, NULL, sizeof(fs));
      memset(&pics, NULL, sizeof(pics));
      GetPrivateProfileString("Directory", "FileName", "KittyDir.Ini", fs.FileName, 144, IniFile);
      SetFileStruct(fs);
      mciSendString("open cats.wav alias cats", NULL, NULL, NULL);
      ccs.hWindowMenu = GetSubMenu(hMenu, 1);
      ccs.idFirstChild = 200;
      hMDIClientWnd = CreateWindow("MDICLIENT", NULL, WS_CHILD | WS_CLIPCHILDREN | WS_VISIBLE, 0, 0, 0, 0, hWnd, 0, dllInstance, (LPSTR) &ccs);
      ShowWindow(hMDIClientWnd, SW_SHOW);
      InitializeServer(hWnd, -1);
      ghWnd = hWnd;
      return 0;
    }
    case WM_SETFOCUS :
    {
      if (pics.hPalette1)
	RefreshPicPal(hWnd, pics.hPalette1);
      return DefFrameProc (hWnd, hMDIClientWnd, WM_SETFOCUS, wParam, lParam);
    }
    case WM_CLOSE :
    {
      Sound(MB_ICONQUESTION);
      if (MessageBox(hWnd, "Are You Sure You Want To Exit?", "Exit", MB_YESNO | MB_ICONQUESTION) == IDYES)
      { 
	DestroyWindow(hWnd);
      }
      return 0;
    }
    case WM_PAINTCLIPBOARD :
    {
      PAINTSTRUCT *pps = (PAINTSTRUCT FAR*) LOWORD(lParam);
      char *Text = "KittyCat! Comm Directory\0";
      SetTextColor(pps->hdc, RGB(128, 128, 128));
      SetBkColor(pps->hdc, RGB(0, 0, 0));
      TextOut(pps->hdc, pps->rcPaint.left, pps->rcPaint.top, Text, strlen(Text));
      return 0;
    }
    case WM_DESTROY :
    {
      EndProgram();
      memset(&fs, NULL, sizeof(fs));
      memset(&pics, NULL, sizeof(pics));
      DeletePalette(pics.hPalette1);
      DeleteObject(pics.hBitmap1);
      DeleteObject(pics.hPalette1);
      mciSendString("close cats", NULL, NULL, NULL);
      ClosePort();
      PostQuitMessage(0);
      return 0;
    }
    case WM_COMMAND :
    {
      return HANDLE_WM_COMMAND(hWnd, wParam, lParam, WMCommand_Handler);
    }
  }
  return DefFrameProc (hWnd, hMDIClientWnd, message, wParam, lParam);
}

#pragma argsused
void FAR PASCAL _export WMCommand_Handler(HWND hWnd, int id, HWND hwndCtl, UINT codeNotify)
{
  switch (id)
  {
    case 100 :
    {
      // Sounds Switch
      if (PlaySounds == TRUE)
      {
	PlaySounds = FALSE;
        CheckMenuItem(hMenu, 100, MF_UNCHECKED);
	WritePrivateProfileString("Setup", "Sounds", "off", IniFile);
        break;
      }
      if (PlaySounds == FALSE)
      {
	PlaySounds = TRUE;
	CheckMenuItem(hMenu, 100, MF_CHECKED);
	WritePrivateProfileString("Setup", "Sounds", "on", IniFile);
	break;
      }
    }
    case 101 :
    {
      // New Directory
      char NewFile[144];
      memset(NewFile, NULL, 144);
      FileDialogRunner(hWnd, NewFile, "New Directory Name", fs.FileName, "INI-Dir~*.ini~All Files~*.*~~", "ini", FALSE, TRUE);
      if (strcmp(NewFile, "") != 0)
      {
	memset(fs.FileName, NULL, 144);
	strcpy(fs.FileName, NewFile);
	CloseMDIChildren();
	SetFileStruct(fs);
	LoadBBSEntriesIntoMDI(fs);
	Sound(MB_ICONQUESTION);
	if (MessageBox(hWnd, "Do you want this directory to load when you load KittyCat! Comm?", "New Directory", MB_YESNO | MB_ICONQUESTION | MB_DEFBUTTON1) == IDYES)
	{
          WritePrivateProfileString("Directory", "FileName", fs.FileName, IniFile);
        }
      }
      break;
    }
    case 102 :
    {
      // Open Directory
      char NewFile[144];
      memset(NewFile, NULL, 144);
      FileDialogRunner(hWnd, NewFile, "Open Directory", fs.FileName, "INI-Dir~*.ini~All Files~*.*~~", "ini", TRUE, FALSE);
      if (strcmp(NewFile, "") != 0)
      {
        memset(fs.FileName, NULL, 144);
	strcpy(fs.FileName, NewFile);
	CloseMDIChildren();
	SetFileStruct(fs);
	LoadBBSEntriesIntoMDI(fs);
	Sound(MB_ICONQUESTION);
	if (MessageBox(hWnd, "Do you want this directory to load when you load KittyCat! Comm?", "Open Directory", MB_YESNO | MB_ICONQUESTION | MB_DEFBUTTON1) == IDYES)
	{
          WritePrivateProfileString("Directory", "FileName", fs.FileName, IniFile);
	}
      }
      break;
    }
    case 103 :
    {
      // Delete Directory
      Sound(MB_ICONEXCLAMATION);
      if (MessageBox(hWnd, "Are you sure you want to delete this directory?!  It can only be undeleted by an external program!  After you delete this directory, the directory will be set to \"KittyDir.Ini\".", "Delete Directory", MB_YESNO | MB_ICONEXCLAMATION | MB_DEFBUTTON2) == IDYES)
      {
        OFSTRUCT os;
	if (OpenFile(fs.FileName, &os, OF_DELETE) == HFILE_ERROR)
	{
	  Sound(MB_ICONEXCLAMATION);
          MessageBox(hWnd, "The directory could not be deleted!  Be sure your access to the file is not read-only and is deletable if on a LAN.", "Delete Error", MB_OK | MB_ICONEXCLAMATION);
	}
	else
	{
          char OldFile[144];
          GetPrivateProfileString("Directory", "FileName", "KittyDir.Ini", OldFile, 144, IniFile);
	  if ((strcmp(OldFile, fs.FileName) == 0) && (strcmp(OldFile, "KittyDir.Ini") != 0))
	    WritePrivateProfileString("Directory", "FileName", "KittyDir.Ini", IniFile);
	  memset(fs.FileName, NULL, 144);
          strcpy(fs.FileName, "KittyDir.Ini");
	  CloseMDIChildren();
	  SetFileStruct(fs);
	  LoadBBSEntriesIntoMDI(fs);
	}
      }
      break;
    }
    case 105 :
    {
      // Merge Directories
      break;
    }
    case 106 :
    {
      // Compact Directory
      break;
    }
    case 107 :
    {
      // Exit
      Sound(MB_ICONQUESTION);
      if (MessageBox(hWnd, "Are You Sure You Want To Exit?", "Exit", MB_YESNO | MB_ICONQUESTION) == IDYES)
      { 
	DestroyWindow(hWnd);
      }
      break;
    } 
    case 108 :
    {
      // Create a new BBS
      NewBBS(hWnd, fs);
      break;
    }
    case 109 :
    {
      //Dial Last Number
      break;
    }
    case 110 :
    {
      // Hang Up
/*      Sound(MB_ICONQUESTION);
      if (MessageBox(hWnd, "Are You Sure You Want To Hang Up?", "Hangup", MB_YESNO | MB_ICONQUESTION | MB_DEFBUTTON2) == IDYES)
      {
	ds = CloseCommPort(ds);
        if (strcmp(ds.ErrorString, "") != 0)
        {
	  Sound(MB_ICONSTOP);
	  MessageBox(hWnd, ds.ErrorString, "Communications Failure", MB_OK | MB_ICONSTOP);
	}
	EnableMenuItem(hMenu, 110, MF_GRAYED);
      }*/
      break;
    }
    case 206 :
    {
      // Configure the Modem
      ModemConfig(hWnd);
      break;
    }
    case 207 :
    {
      // Result Messages
      ResultMessages(hWnd);
      break;
    }
    case 214 :
    {
      // Index
      break;
    }
    case 215 :
    {
      // Using
      break;
    }
    case 216 :
    {
      // Configuring
      break;
    }
    case 217 :
    {
      // BBS Help
      break;
    }
    case 218 :
    {
      // About KittyCat! Comm
      About(hWnd);
      break;
    }
    case 400 :
    {
      // Arrange Icons
      SendMessage(hMDIClientWnd, WM_MDIICONARRANGE, NULL, NULL);
      break;
    }
    case 401 :
    {
      // Sort List
      break;
    }
    default :
    {
      DefFrameProc(hWnd, hMDIClientWnd, WM_COMMAND, id, 0L);
    }
  }
}


// Program Functions
DWORD FAR PASCAL _export IsVersionRegistered(void)
{
  // Registered Version
  return TRUE;
}

HWND FAR PASCAL _export GetGlobalWindow(void)
{
  return ghWnd;
}

void FAR PASCAL _export Sound(UINT AlertLevel)
{
  if (PlaySounds == TRUE)
  {
    MessageBeep(AlertLevel);
  }
}

int FAR PASCAL _export GetSoundVariable(void)
{
  int Return;
  if (PlaySounds == TRUE)
    Return = 1;
  if (PlaySounds == FALSE)
    Return = 0;
  return Return;
}

void FAR PASCAL _export WaveFile(LPCSTR String)
{
  if (PlaySounds == TRUE)
  {
    mciSendString(String, NULL, NULL, NULL);
  } 
}

void FAR PASCAL _export ChangeCursor(HWND hWnd, LPSTR hCursor)
{
  SetClassWord(hWnd, GCW_HCURSOR, (WORD) LoadCursor(ResourceInstance, hCursor));
  SetCursor(LoadCursor(ResourceInstance, hCursor));
}

RESULT FAR PASCAL _export LoadResultTable(RESULT Result)
{
  memset(&Result, NULL, sizeof(Result));
  GetPrivateProfileString("Result Messages", "Connect1200", "CONNECT 1200", Result.Connect1200, 50, IniFile);
  GetPrivateProfileString("Result Messages", "Connect2400", "CONNECT 2400", Result.Connect2400, 50, IniFile);
  GetPrivateProfileString("Result Messages", "Connect4800", "CONNECT 4800", Result.Connect4800, 50, IniFile);
  GetPrivateProfileString("Result Messages", "Connect9600", "CONNECT 9600", Result.Connect9600, 50, IniFile);
  GetPrivateProfileString("Result Messages", "Connect14400", "CONNECT 14400", Result.Connect14400, 50, IniFile);
  GetPrivateProfileString("Result Messages", "Connect19200", "CONNECT 19200", Result.Connect19200, 50, IniFile);
  GetPrivateProfileString("Result Messages", "Connect38400", "CONNECT 38400", Result.Connect38400, 50, IniFile);
  GetPrivateProfileString("Result Messages", "Connect57600", "CONNECT 57600", Result.Connect57600, 50, IniFile);
  GetPrivateProfileString("Result Messages", "Connect115200", "CONNECT 115200", Result.Connect115200, 50, IniFile);
  GetPrivateProfileString("Result Messages", "Connect256000", "CONNECT 256000", Result.Connect256000, 50, IniFile);
  GetPrivateProfileString("Result Messages", "NoConnect1", "BUSY", Result.NoConnect1, 50, IniFile);
  GetPrivateProfileString("Result Messages", "NoConnect2", "NO DIALTONE", Result.NoConnect2, 50, IniFile);
  GetPrivateProfileString("Result Messages", "NoConnect3", "NO ANSWER", Result.NoConnect3, 50, IniFile);
  GetPrivateProfileString("Result Messages", "NoConnect4", "ERROR", Result.NoConnect4, 50, IniFile);
  GetPrivateProfileString("Result Messages", "NoConnect5", "NO CARRIER", Result.NoConnect5, 50, IniFile);
  GetPrivateProfileString("Result Messages", "NoConnect6", "", Result.NoConnect6, 50, IniFile);
  return Result;
}

void FAR PASCAL _export SaveResultTable(RESULT Result)
{
  WritePrivateProfileString("Result Messages", "Connect1200", Result.Connect1200, IniFile);
  WritePrivateProfileString("Result Messages", "Connect2400", Result.Connect2400, IniFile);
  WritePrivateProfileString("Result Messages", "Connect4800", Result.Connect4800, IniFile);
  WritePrivateProfileString("Result Messages", "Connect9600", Result.Connect9600, IniFile);
  WritePrivateProfileString("Result Messages", "Connect14400", Result.Connect14400, IniFile);
  WritePrivateProfileString("Result Messages", "Connect19200", Result.Connect19200, IniFile);
  WritePrivateProfileString("Result Messages", "Connect38400", Result.Connect38400, IniFile);
  WritePrivateProfileString("Result Messages", "Connect57600", Result.Connect57600, IniFile);
  WritePrivateProfileString("Result Messages", "Connect115200", Result.Connect115200, IniFile);
  WritePrivateProfileString("Result Messages", "Connect256000", Result.Connect256000, IniFile);
  WritePrivateProfileString("Result Messages", "NoConnect1", Result.NoConnect1, IniFile);
  WritePrivateProfileString("Result Messages", "NoConnect2", Result.NoConnect2, IniFile);
  WritePrivateProfileString("Result Messages", "NoConnect3", Result.NoConnect3, IniFile);
  WritePrivateProfileString("Result Messages", "NoConnect4", Result.NoConnect4, IniFile);
  WritePrivateProfileString("Result Messages", "NoConnect5", Result.NoConnect5, IniFile);
  WritePrivateProfileString("Result Messages", "NoConnect6", Result.NoConnect6, IniFile);
}

DATASTRUCT FAR PASCAL _export LoadDefaultStruct(DATASTRUCT DataStruct)
{
  char WorkStr[300];
  memset(&DataStruct, NULL, sizeof(DataStruct));
  GetPrivateProfileString("Setup", "CommPort", "COM1", WorkStr, 10, IniFile);
  strcpy(DataStruct.CommString, WorkStr);
  strcpy(DataStruct.CommPort, WorkStr);
  strcat(DataStruct.CommString, ":");
  GetPrivateProfileString("Setup", "CommSpeed", "14400", WorkStr, 10, IniFile);
  strcat(DataStruct.CommString, "9600");
  strcat(DataStruct.CommString, ",");
  DataStruct.CommSpeed = atol(WorkStr);
  GetPrivateProfileString("Setup", "CommParity", "N", WorkStr, 10, IniFile);
  strcpy(DataStruct.CommParity, WorkStr);
  strlwr(WorkStr);
  strcat(DataStruct.CommString, WorkStr);
  strcat(DataStruct.CommString, ",");
  GetPrivateProfileString("Setup", "CommByteSize", "8", WorkStr, 10, IniFile);
  strcat(DataStruct.CommString, WorkStr);
  strcat(DataStruct.CommString, ",");
  DataStruct.CommByteSize = atoi(WorkStr);
  GetPrivateProfileString("Setup", "CommStopBits", "10", WorkStr, 10, IniFile);
  strcat(DataStruct.CommString, WorkStr);
  DataStruct.CommStopBits = atof(WorkStr);
  DataStruct.Volume = GetPrivateProfileInt("Setup", "VolumeLevel", 0, IniFile);
  GetPrivateProfileString("Setup", "InitString", "ATZ0^M", DataStruct.InitString, 100, IniFile);
  GetPrivateProfileString("Setup", "DialPrefix", "ATDT", DataStruct.DialPrefix, 50, IniFile);
  GetPrivateProfileString("Setup", "DialSuffix", "^M", DataStruct.DialSuffix, 50, IniFile);
  GetPrivateProfileString("Setup", "HangupString", "+++ATH0^M", DataStruct.HangupString, 50, IniFile);
  GetPrivateProfileString("Setup", "ErrorCorrection", "not_initialized", WorkStr, 30, IniFile);
  if (strcmp(WorkStr, "on") == 0)
  {
    DataStruct.CommEC = 1;
  }
  if (strcmp(WorkStr, "off") == 0)
  {
    DataStruct.CommEC = 0;
  }
  if (strcmp(WorkStr, "not_initialized") == 0)
  {
    DataStruct.CommEC = 2;
  }
  return DataStruct;
}

void FAR PASCAL _export SaveDefaultStruct(DATASTRUCT DataStruct)
{
  char WorkStr[301];
  WritePrivateProfileString("Setup", "CommPort", DataStruct.CommPort, IniFile);
  gcvt(DataStruct.CommSpeed, 10, WorkStr);
  WritePrivateProfileString("Setup", "CommSpeed", WorkStr, IniFile);
  WritePrivateProfileString("Setup", "CommParity", DataStruct.CommParity, IniFile);
  gcvt(DataStruct.CommByteSize, 10, WorkStr);
  WritePrivateProfileString("Setup", "CommByteSize", WorkStr, IniFile);
  gcvt(DataStruct.CommStopBits, 10, WorkStr);
  WritePrivateProfileString("Setup", "CommStopBits", WorkStr, IniFile);
  gcvt(DataStruct.Volume, 10, WorkStr);
  WritePrivateProfileString("Setup", "VolumeLevel", WorkStr, IniFile);
  WritePrivateProfileString("Setup", "InitString", DataStruct.InitString, IniFile);
  WritePrivateProfileString("Setup", "DialPrefix", DataStruct.DialPrefix, IniFile);
  WritePrivateProfileString("Setup", "DialSuffix", DataStruct.DialSuffix, IniFile);
  WritePrivateProfileString("Setup", "HangupString", DataStruct.HangupString, IniFile);
  memset(WorkStr, NULL, strlen(WorkStr));
  if (DataStruct.CommEC == 1)
  {
    strcpy(WorkStr, "on");
  }
  if (DataStruct.CommEC == 0)
  {
    strcpy(WorkStr, "off");
  }
  if (DataStruct.CommEC == 2)
  {
    strcpy(WorkStr, "not_initialized");
  }
  WritePrivateProfileString("Setup", "ErrorCorrection", WorkStr, IniFile);
}

void FAR PASCAL _export SetMainWindowPics(PICS Pics)
{
  pics = Pics;
}

HWND FAR PASCAL _export MakeNewChild(char *pName)
{
   HWND hWnd;
   MDICREATESTRUCT mcs;
   if (!pName)
      mcs.szTitle = "Internal Error: No Name";
   else
      mcs.szTitle = (LPSTR)pName;
   mcs.szClass  = "DialMDI";
   mcs.hOwner   = dllInstance;
   mcs.x        = 20;
   mcs.y        = 20;
   mcs.cx       = 21;
   mcs.cy       = 21;
   mcs.style    = WS_MINIMIZE | WS_DLGFRAME;
   hWnd = (HWND)SendMessage(hMDIClientWnd, WM_MDICREATE, 0,
			   (LONG)(LPMDICREATESTRUCT)&mcs);
   ShowWindow(hWnd, SW_SHOWMINIMIZED);
   return hWnd;
}

LRESULT CALLBACK _export MDIChildWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
  switch(message)
  {
    case WM_CREATE :
    {
      // Window Is Being Created
      HMENU hSysMenu;
      hSysMenu = GetSystemMenu(hWnd, FALSE);
      ModifyMenu(hSysMenu, SC_RESTORE, MF_STRING | MF_BYCOMMAND, 301, "&Dial");
      ModifyMenu(hSysMenu, SC_MAXIMIZE, MF_STRING | MF_BYCOMMAND, SC_MAXIMIZE, "&Configure");
      ModifyMenu(hSysMenu, SC_CLOSE, MF_STRING | MF_BYCOMMAND, SC_CLOSE, "D&elete");
      DeleteMenu(hSysMenu, SC_MOVE, MF_BYCOMMAND);
      DeleteMenu(hSysMenu, SC_MINIMIZE, MF_BYCOMMAND);
      DeleteMenu(hSysMenu, SC_SIZE, MF_BYCOMMAND);
      DeleteMenu(hSysMenu, SC_TASKLIST, MF_BYCOMMAND);
      InsertMenu(hSysMenu, 5, MF_BYPOSITION | MF_STRING, SC_MOVE, "&Move Icon");
      InsertMenu(hSysMenu, 2, MF_BYPOSITION | MF_STRING, 300, "&Name");
      InsertMenu(hSysMenu, 3, MF_BYPOSITION | MF_STRING, SC_RESTORE, "&BBS Options");
      InsertMenu(hSysMenu, 5, MF_BYPOSITION | MF_STRING, 304, "&Paste\tShift+Ins");
      InsertMenu(hSysMenu, 5, MF_BYPOSITION | MF_STRING, 303, "&Copy\tCtrl+Ins");
      InsertMenu(hSysMenu, 5, MF_BYPOSITION | MF_STRING, 302, "Cu&t\tShift+Del");
      return DefMDIChildProc(hWnd, WM_CREATE, wParam, lParam);
    }
    case WM_PAINT :
    {
      // Show BBS Options Dialog
      ShowWindow(hWnd, SW_SHOWMINIMIZED);
      BBSOptions(hWnd, fs);
      if (AutoArrange == TRUE)
	SendMessage(hMDIClientWnd, WM_MDIICONARRANGE, NULL, NULL);
      SendMessage(hMDIClientWnd, WM_MDISETMENU, TRUE, NULL);
      return 0;
    }
    case WM_CLOSE :
    {
      // Delete Entry
      char TmpStr[301];
      char TextStr[201];
      GetWindowText(hWnd, TextStr, 200);
      strcpy(TmpStr, "Are you sure you want to delete \"");
      strcat(TmpStr, TextStr);
      strcat(TmpStr, "\"?");
      Sound(MB_ICONEXCLAMATION);
      if (MessageBox(hWnd, TmpStr, "Delete BBS Entry", MB_YESNO | MB_DEFBUTTON2 | MB_ICONEXCLAMATION) == IDYES)
      {
	DeleteBBS(hWnd, fs);
	DefMDIChildProc(hWnd, WM_CLOSE, wParam, lParam);
	if (AutoArrange == TRUE)
	  SendMessage(hMDIClientWnd, WM_MDIICONARRANGE, NULL, NULL);
      }
      return 0;
    }
    case WM_SYSCOMMAND :
    {
      switch (wParam)
      {
	case SC_RESTORE :
	{
	  // Show BBS Options Dialog
          ShowWindow(hWnd, SW_SHOWMINIMIZED);
          BBSOptions(hWnd, fs);
	  return 0;
        }
	case 301 :
	{
          // Dial Number
	  int LineNumber = 1;
	  BBS bbs;
	  memset(&bbs, NULL, sizeof(BBS));
          GetWindowText(hWnd, bbs.BBSName, 50);
          DialNumber(hWnd, fs, bbs, LineNumber);
	  return 0;
        }
	case SC_MAXIMIZE :
	{
          // Configuration
	  DialDirConfig(hWnd, fs);
	  return 0;
        }   
	case 300 :
	{
          // Rename BBS
	  RenameEntry(hWnd, fs);
	  return 0;
        }
	case 302 :
	{
          // Cut BBS
          CutBBS(ghWnd);
	  return 0;
        }
	case 303 :
	{
          // Copy BBS
	  char BBSName[50];
	  memset(BBSName, NULL, 50);
          GetWindowText(hWnd, BBSName, 50);
	  CopyBBS(ghWnd, BBSName);
	  return 0;
        }
	case 304 :
	{
          // Paste BBS
	  PasteBBS(ghWnd);
	  return 0;
        }
      }
      return DefMDIChildProc(hWnd, WM_SYSCOMMAND, wParam, lParam);
    }
/*    case WM_ICONERASEBKGND :
    {
      // Arrange Icons
      if (AutoArrange == TRUE)
	SendMessage(hMDIClientWnd, WM_MDIICONARRANGE, NULL, NULL);
      return DefMDIChildProc(hWnd, WM_ICONERASEBKGND, wParam, lParam);
    }*/
/*    case WM_WINDOWPOSCHANGED :
    {
      // Arrange Icons
      if (AutoArrange == TRUE)
	SendMessage(hMDIClientWnd, WM_MDIICONARRANGE, NULL, NULL);
      return DefMDIChildProc(hWnd, WM_ICONERASEBKGND, wParam, lParam);
    }*/
  }
  return DefMDIChildProc(hWnd, message, wParam, lParam);
}

void FileDialogRunner(HWND hWnd, char WorkStr[144], char *DialogText, char *DefaultText, char *FileString, char *DefExt, DWORD MustExist, DWORD OverwritePrompt)
{
  char NewFileString[144];
  OPENFILENAME ofn;
  char File[144];
  char Dir[144];
  int i;
  for (i=0;i<=strlen(FileString);i++)
  {
    NewFileString[i]=FileString[i];
    if (FileString[i]=='~')
    {
      NewFileString[i] = '\0';
    }
  }
  if (strcmp(DefaultText, "") == 0)
  {
    Dir[0] = '\0';
    File[0] = '\0';
    DefaultText[0] = '\0';
  }
  else
  {
    int i, t, s;
    strrev(DefaultText);
    for (i=0;i<=strlen(DefaultText);i++)
    {
      if (DefaultText[i] == '\\')
      {
        t = i+1;
        File[i] = '\0';
	goto GetDir;
      }
      if (DefaultText[i] == ':')
      {
	t = i+1;
	File[i] = '\0';
	goto GetDir;
      }
      else
      {
        File[i] = DefaultText[i];
      }
    }
    GetDir:
    for (s=t;s<=strlen(DefaultText);s++)
    {
      Dir[s-t] = DefaultText[s];
    }
    Dir[strlen(DefaultText)-t] = '\0';
    strrev(Dir);
    strrev(File);
    strrev(DefaultText);
  }
  memset(&ofn, 0, sizeof(OPENFILENAME));
  ofn.lStructSize = sizeof(OPENFILENAME);
  ofn.hwndOwner = hWnd;
  ofn.lpstrFilter = NewFileString;
  ofn.lpstrFile = File;
  ofn.nMaxFile = 144;
  ofn.lpstrFileTitle = WorkStr;
  ofn.nMaxFileTitle = 144;
  ofn.lpstrInitialDir = Dir;
  ofn.lpstrDefExt = DefExt;
  ofn.lpstrTitle = DialogText;
  if ((MustExist == TRUE) && (OverwritePrompt == FALSE))
    ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY;
  if ((MustExist == FALSE) && (OverwritePrompt == TRUE))
    ofn.Flags = OFN_PATHMUSTEXIST | OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT;
  if ((MustExist == FALSE) && (OverwritePrompt == FALSE))
    ofn.Flags = OFN_PATHMUSTEXIST | OFN_HIDEREADONLY;
  if (OverwritePrompt == TRUE)
    if (!GetSaveFileName(&ofn))
      strcpy(WorkStr, "");
    else
      strcpy(WorkStr, ofn.lpstrFile);
  if (OverwritePrompt == FALSE)
    if (!GetOpenFileName(&ofn))
      strcpy(WorkStr, "");
    else
      strcpy(WorkStr, ofn.lpstrFile);
}

void CloseMDIChildren(void)
{
  HWND hWndTemp;
  hWndTemp = GetWindow(hMDIClientWnd, GW_CHILD);
  while (hWndTemp)
  {
    hWndTemp = GetWindow(hMDIClientWnd, GW_CHILD);
    while (hWndTemp && GetWindow(hWndTemp, GW_OWNER))
      hWndTemp = GetWindow(hWndTemp, GW_HWNDNEXT);
    if (hWndTemp)
      SendMessage(hMDIClientWnd, WM_MDIDESTROY, (WORD) hWndTemp, 0L);
    else
      break;
  }
}

void CutBBS(HWND hWnd)
{
}

void CopyBBS(HWND hWnd, LPCSTR BBSName)
{
}

void PasteBBS(HWND hWnd)
{
}
